/* This file is part of Netwib.
*/

/*-------------------------------------------------------------*/
static netwib_err netwib_priv_sa_info(const netwib_priv_sockaddr_noalign *psa,
                                      netwib_uint32 salen,
                                      netwib_uint32 *psafamily,
                                      netwib_uint32 *psalen)
{
  struct sockaddr ssa;
  netwib_uint32 safamily, sizetocopy;

  /* copy only beginning (only used here) */
#if defined NETWIBDEF_SYSNAME_BSD
  sizetocopy = sizeof(ssa.sa_family) + sizeof(ssa.sa_len);
#else
  sizetocopy = sizeof(ssa.sa_family);
#endif
  netwib_c_memcpy(&ssa, psa, sizetocopy);

  /* because sa_family is not always set */
  safamily = ssa.sa_family;
#if defined NETWIBDEF_SYSNAME_BSD
  if (safamily == 0) {
    if (ssa.sa_len == sizeof(struct sockaddr_in)) {
      safamily = AF_INET;
 #if NETWIBDEF_HAVETYPE_SOCKADDR_IN6 == 1
  #if NETWIBDEF_HAVEVAR_AF_INET6 == 1
    } else if (ssa.sa_len == sizeof(struct sockaddr_in6)) {
      safamily = AF_INET6;
  #endif
 #endif
    /* Those values are very stupid, but they are encountered
       when reading device/route configuration under FreeBSD 4.0 */
    } else if (ssa.sa_len >= 5 && ssa.sa_len <= 8) {
      safamily = AF_INET;
 #if NETWIBDEF_HAVEVAR_AF_INET6 == 1
    } else if (ssa.sa_len == 0x1C) {
      safamily = AF_INET6;
 #endif
    } else if (ssa.sa_len == 0) {
      salen = 0;
    }
    /* end of stupid values */
  }
  if (salen == 0) {
    salen = ssa.sa_len;
  }
#endif

  switch(safamily) {
  case AF_INET :
    if (salen == 0 || salen > sizeof(struct sockaddr_in)) {
      salen = sizeof(struct sockaddr_in);
    }
    break;
#if NETWIBDEF_HAVETYPE_SOCKADDR_IN6 == 1
 #if NETWIBDEF_HAVEVAR_AF_INET6 == 1
  case AF_INET6 :
    if (salen == 0 || salen > sizeof(struct sockaddr_in6)) {
      salen = sizeof(struct sockaddr_in6);
    }
    break;
 #endif
#endif
  default :
    return(NETWIB_ERR_SPIPTYPE);
    break;
  }

  *psafamily = safamily;
  *psalen = salen;
  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_sa_ipport_init_sal(const netwib_priv_sockaddr *psa,
                                          netwib_uint32 salen,
                                          netwib_ip *pip,
                                          netwib_port *pport)
{
  const struct sockaddr_in *psain;
#if NETWIBDEF_HAVETYPE_SOCKADDR_IN6 == 1
  const struct sockaddr_in6 *psain6;
#endif
  netwib_uint32 safamily;

  netwib_er(netwib_priv_sa_info((const netwib_priv_sockaddr_noalign*)psa,
                                salen, &safamily, &salen));

  if (salen == 0) {
    if (pip != NULL) {
      pip->iptype = NETWIB_IPTYPE_IP4;
      pip->ipvalue.ip4 = 0;
    }
    if (pport != NULL) {
      *pport = 0;
    }
    return(NETWIB_ERR_OK);
  }

  switch(safamily) {
  case AF_INET :
    psain = (const struct sockaddr_in*)psa;
    if (pip != NULL) {
      pip->iptype = NETWIB_IPTYPE_IP4;
      pip->ipvalue.ip4 = netwib_priv_ntohl(psain->sin_addr.s_addr);
    }
    if (pport != NULL) {
      *pport = netwib_priv_ntohs(psain->sin_port);
    }
    break;
#if NETWIBDEF_HAVETYPE_SOCKADDR_IN6 == 1
 #if NETWIBDEF_HAVEVAR_AF_INET6 == 1
  case AF_INET6 :
    psain6 = (const struct sockaddr_in6*)psa;
    if (pip != NULL) {
      pip->iptype = NETWIB_IPTYPE_IP6;
      netwib_c_memcpy(pip->ipvalue.ip6.b, psain6->sin6_addr.s6_addr,
                      NETWIB_IP6_LEN);
  #if defined NETWIBDEF_SYSNAME_BSD
      if (pip->ipvalue.ip6.b[0] == 0xFE && pip->ipvalue.ip6.b[1] == 0x80) {
        /* under FreeBSD, this field contains device number : reset it */
        pip->ipvalue.ip6.b[3] = 0;
      }
  #endif
    }
    if (pport != NULL) {
      *pport = netwib_priv_ntohs(psain6->sin6_port);
    }
    break;
 #endif
#endif
  default :
    return(NETWIB_ERR_SPIPTYPE);
    break;
  }

  return(NETWIB_ERR_OK);
#if NETWIBDEF_HAVETYPE_SOCKADDR_IN6 == 1
  psain6 = NULL;
#endif
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_sa_ipport_init_saln(const netwib_priv_sockaddr_noalign *psa,
                                           netwib_uint32 salen,
                                           netwib_ip *pip,
                                           netwib_port *pport)
{
  struct sockaddr_in sain;
#if NETWIBDEF_HAVETYPE_SOCKADDR_IN6 == 1
  struct sockaddr_in6 sain6;
#endif
  netwib_uint32 safamily;

  netwib_er(netwib_priv_sa_info(psa, salen, &safamily, &salen));

  if (salen == 0) {
    if (pip != NULL) {
      pip->iptype = NETWIB_IPTYPE_IP4;
      pip->ipvalue.ip4 = 0;
    }
    if (pport != NULL) {
      *pport = 0;
    }
    return(NETWIB_ERR_OK);
  }

  switch(safamily) {
  case AF_INET :
    netwib_c_memset(&sain, 0, sizeof(struct sockaddr_in));
    netwib_c_memcpy(&sain, psa, salen);
    if (pip != NULL) {
      pip->iptype = NETWIB_IPTYPE_IP4;
      pip->ipvalue.ip4 = netwib_priv_ntohl(sain.sin_addr.s_addr);
    }
    if (pport != NULL) {
      *pport = netwib_priv_ntohs(sain.sin_port);
    }
    break;
#if NETWIBDEF_HAVETYPE_SOCKADDR_IN6 == 1
 #if NETWIBDEF_HAVEVAR_AF_INET6 == 1
  case AF_INET6 :
    netwib_c_memset(&sain6, 0, sizeof(struct sockaddr_in6));
    netwib_c_memcpy(&sain6, psa, salen);
    if (pip != NULL) {
      pip->iptype = NETWIB_IPTYPE_IP6;
      netwib_c_memcpy(pip->ipvalue.ip6.b, sain6.sin6_addr.s6_addr,
                      NETWIB_IP6_LEN);
  #if defined NETWIBDEF_SYSNAME_BSD
      if (pip->ipvalue.ip6.b[0] == 0xFE && pip->ipvalue.ip6.b[1] == 0x80) {
        /* under FreeBSD, this field contains device number : reset it */
        pip->ipvalue.ip6.b[3] = 0;
      }
  #endif
    }
    if (pport != NULL) {
      *pport = netwib_priv_ntohs(sain6.sin6_port);
    }
    break;
 #endif
#endif
  default :
    return(NETWIB_ERR_SPIPTYPE);
    break;
  }

  return(NETWIB_ERR_OK);
#if NETWIBDEF_HAVETYPE_SOCKADDR_IN6 == 1
  sain6.sin6_port = 0;
#endif
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_sa_sal_init_iptport(netwib_constip *pip,
                                           netwib_iptype iptype,
                                           netwib_port port,
                                           netwib_priv_sockaddr *psa,
                                           netwib_uint32 *psalen)
{
  struct sockaddr_in *psain;
#if NETWIBDEF_HAVETYPE_SOCKADDR_IN6 == 1
  struct sockaddr_in6 *psain6;
#endif

  if (pip != NULL) {
    iptype = pip->iptype;
  }

  switch(iptype) {
  case NETWIB_IPTYPE_IP4 :
    if (psa != NULL) {
      netwib_c_memset(psa, 0, sizeof(struct sockaddr_in));
      psain = (struct sockaddr_in *)psa;
 #if defined NETWIBDEF_SYSNAME_BSD
      psain->sin_len = sizeof(struct sockaddr_in);
 #endif
      psain->sin_family = AF_INET;
      psain->sin_port = netwib_priv_htons(port);
      if (pip != NULL) {
        psain->sin_addr.s_addr = netwib_priv_htonl(pip->ipvalue.ip4);
      } else {
        psain->sin_addr.s_addr = netwib_priv_htonl(INADDR_ANY);
      }
    }
    if (psalen != NULL) {
      *psalen = sizeof(struct sockaddr_in);
    }
    break;
#if NETWIBDEF_HAVETYPE_SOCKADDR_IN6 == 1
 #if NETWIBDEF_HAVEVAR_AF_INET6 == 1
  case NETWIB_IPTYPE_IP6 :
    if (psa != NULL) {
      netwib_c_memset(psa, 0, sizeof(struct sockaddr_in6));
      psain6 = (struct sockaddr_in6 *)psa;
 #if defined NETWIBDEF_SYSNAME_BSD
      psain6->sin6_len = sizeof(struct sockaddr_in6);
 #endif
      psain6->sin6_family = AF_INET6;
      psain6->sin6_port = netwib_priv_htons(port);
      if (pip != NULL) {
        netwib_c_memcpy(psain6->sin6_addr.s6_addr, pip->ipvalue.ip6.b,
                        NETWIB_IP6_LEN);
      } else {
        netwib_c_memset(psain6->sin6_addr.s6_addr, 0, NETWIB_IP6_LEN);
      }
    }
    if (psalen != NULL) {
      *psalen = sizeof(struct sockaddr_in6);
    }
    break;
 #endif
#endif
  default :
    return(NETWIB_ERR_SPIPTYPE);
    break;
  }

  return(NETWIB_ERR_OK);
#if NETWIBDEF_HAVETYPE_SOCKADDR_IN6 == 1
  psain6 = NULL;
#endif
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_sa_send(int fd,
                               netwib_constbuf *pbuf)
{
  netwib_data data;
  netwib_uint32 datasize;
  int reti, sendopt;

  data = netwib__buf_ref_data_ptr(pbuf);
  datasize = netwib__buf_ref_data_size(pbuf);

#if defined MSG_NOSIGNAL
  sendopt = MSG_NOSIGNAL;
#else
  sendopt = 0;
#endif

  reti = netwib_c_send(fd, data, datasize, sendopt);
  if (reti == -1) {
    if (errno == EBADF) {
      errno = 0;
      return(NETWIB_ERR_LOOBJWRITENOTSUPPORTED);
    }
    return(NETWIB_ERR_FUSEND);
  }

  if ((netwib_uint32)reti != datasize) {
    return(NETWIB_ERR_FUSEND);
  }

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_sa_sendto(int fd,
                                 netwib_constbuf *pbuf,
                                 const netwib_priv_sockaddr *psa,
                                 netwib_uint32 salen)
{
  netwib_data data;
  netwib_uint32 datasize;
  int reti;

  data = netwib__buf_ref_data_ptr(pbuf);
  datasize = netwib__buf_ref_data_size(pbuf);

  reti = netwib_c_sendto(fd, data, datasize, 0,
                         (const struct sockaddr*)psa, salen);
  if (reti == -1) {
    if (errno == EBADF) {
      errno = 0;
      return(NETWIB_ERR_LOOBJWRITENOTSUPPORTED);
    }
    return(NETWIB_ERR_FUSENDTO);
  }

  if ((netwib_uint32)reti != datasize) {
    return(NETWIB_ERR_FUSENDTO);
  }

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_sa_recv(int fd,
                               netwib_buf *pbuf)
{
  netwib_data data;
  netwib_uint32 maxmsgsize;
  int readsize;

  netwib_er(netwib_buf_wishspace(pbuf, NETWIB_PRIV_MINMSGSIZE_READ,
                                 &data, &maxmsgsize));
  if (maxmsgsize == 0) {
    return(NETWIB_ERR_OKDATANOSPACE);
  }

  /* read */
  readsize = netwib_c_recv(fd, data, maxmsgsize, 0);
  /* an error occured */
#if defined NETWIBDEF_SYSNAME_Unix
  if (readsize == -1) {
    if (errno == EAGAIN) {
      /* nothing was available */
      errno = 0;
      return(NETWIB_ERR_OKDATANOTAVAIL);
    } else if (errno == EBADF) {
      /* reached end */
      errno = 0;
      return(NETWIB_ERR_OKDATAEND);
    } else {
      /* fatal error */
      return(NETWIB_ERR_FURECV);
    }
  }
#elif defined NETWIBDEF_SYSNAME_Windows
  if (readsize == SOCKET_ERROR) {
    switch(h_errno) {
    case WSAESHUTDOWN :
    case WSAECONNRESET :
      return(NETWIB_ERR_OKDATAEND);
    case WSAEWOULDBLOCK :
      return(NETWIB_ERR_OKDATANOTAVAIL);
    default :
      return(NETWIB_ERR_FURECV);
    }
  }
#else
 #error "Unknown value for NETWIBDEF_SYSNAME"
#endif

  /* if we read 0, we are at end of file */
  if (readsize == 0) {
    return(NETWIB_ERR_OKDATAEND);
  }

  /* store read data */
  pbuf->endoffset += readsize;

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_sa_recvfrom(int fd,
                                   netwib_buf *pbuf,
                                   netwib_priv_sockaddr *psa,
                                   netwib_uint32 *psalen)
{
  netwib_data data;
  netwib_uint32 maxmsgsize;
  int readsize;

  netwib_er(netwib_buf_wishspace(pbuf, NETWIB_PRIV_MINMSGSIZE_READ,
                                 &data, &maxmsgsize));
  if (maxmsgsize == 0) {
    return(NETWIB_ERR_OKDATANOSPACE);
  }

  if (psalen != NULL) {
    *psalen = sizeof(netwib_priv_sockaddr);
  }

  /* read */
  readsize = netwib_c_recvfrom(fd, data, maxmsgsize, 0,
                               (struct sockaddr*)psa, psalen);
  /* an error occured */
#if defined NETWIBDEF_SYSNAME_Unix
  if (readsize == -1) {
    if (errno == EAGAIN) {
      /* nothing was available */
      errno = 0;
      return(NETWIB_ERR_OKDATANOTAVAIL);
    } else if (errno == EBADF) {
      /* reached end */
      errno = 0;
      return(NETWIB_ERR_OKDATAEND);
    } else {
      /* fatal error */
      return(NETWIB_ERR_FURECVFROM);
    }
  }
#elif defined NETWIBDEF_SYSNAME_Windows
  if (readsize == SOCKET_ERROR) {
    switch(h_errno) {
    case WSAESHUTDOWN :
    case WSAECONNRESET :
      return(NETWIB_ERR_OKDATAEND);
    case WSAEWOULDBLOCK :
      return(NETWIB_ERR_OKDATANOTAVAIL);
    default :
      return(NETWIB_ERR_FURECV);
    }
  }
#else
 #error "Unknown value for NETWIBDEF_SYSNAME"
#endif

  /* if we read 0, we are at end of file */
  if (readsize == 0) {
    return(NETWIB_ERR_OKDATAEND);
  }

  /* store read data */
  pbuf->endoffset += readsize;

  return(NETWIB_ERR_OK);
}
