/*
                                  NETWIB
                             Network library
                Copyright(c) 1999-2004 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Netwib.

  Netwib is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  version 2 as published by the Free Software Foundation.

  Netwib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

#include <netwib/inc/maininc.h>

/*-------------------------------------------------------------*/
static netwib_err netwib_priv_path_canon_searchsmb(netwib_conststring pc,
                                                   netwib_buf *pbufstart,
                                                   netwib_conststring *ppcstartanalyze)
{
  netwib_conststring pcwork;
  netwib_char c;

  /* pc should contain "server[\/]+share[\/]*.*" */

  /* search the server */
  c = *pc;
  if (c == '\\' || c == '/' || c == '\0') {
    /* path is "\\\" or "\\" which is invalid because there is no
       server name */
    return(NETWIB_ERR_SPPATHNOTCLEANED);
  }
  pcwork = pc;
  while (NETWIB_TRUE) {
    c = *pcwork;
    if (c == '\\' || c == '/') {
      break;
    }
    if (c == '\0') {
      /* path is "\\server" which is invalid because there is no share name */
      return(NETWIB_ERR_SPPATHNOTCLEANED);
    }
    pcwork++;
  }
  netwib_er(netwib_buf_append_data((netwib_constdata)pc, pcwork - pc,
                                   pbufstart));
  netwib_er(netwib_buf_append_byte('\\', pbufstart));

  /* skip / or \ */
  while (*pcwork == '\\' || *pcwork == '/') {
    pcwork++;
  }
  if (*pcwork == '\0') {
    /* path is "\\server\", which is invalid because there is no share name */
    return(NETWIB_ERR_SPPATHNOTCLEANED);
  }

  /* now search the share */
  pc = pcwork;
  while (NETWIB_TRUE) {
    c = *pcwork;
    if (c == '\\' || c == '/' || c == '\0') {
      break;
    }
    pcwork++;
  }
  netwib_er(netwib_buf_append_data((netwib_constdata)pc, pcwork - pc,
                                   pbufstart));
  netwib_er(netwib_buf_append_byte('\\', pbufstart));

  /* set output */
  if (ppcstartanalyze != NULL)
    *ppcstartanalyze = pcwork;
  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
static netwib_err netwib_priv_path_canon_goup(netwib_buf *pout,
                                              netwib_bool *paddaslash)
{
  netwib_data data, pc;
  netwib_uint32 datasize;

  /* convert :
     /       to NETWIB_ERR_SPPATHROOTDOTDOT
     .       to ..
     ..      to ../..
     x/..    to x/../..
     x/aa/bb to x/aa
     x       to ..
     /x      to /
  */

  data = netwib__buf_ref_data_ptr(pout);
  datasize = netwib__buf_ref_data_size(pout);

  if (datasize == 1) {
    if (*data == '/') {
      return(NETWIB_ERR_SPPATHROOTDOTDOT);
    }
    if (*data == '.') {
      /* "." to ".." */
      netwib_er(netwib_buf_append_byte('.', pout));
      return(NETWIB_ERR_OK);
    }
  }

  if (datasize == 2) {
    pc = data;
    if (*pc++ == '.') {
      if (*pc == '.') {
        /* ".." to "../.." */
        netwib_er(netwib_buf_append_text("/..", pout));
        return(NETWIB_ERR_OK);
      }
    }
  }

  if (datasize >= 3) {
    pc = data + datasize - 3;
    if (*pc++ == '/') {
      if (*pc++ == '.') {
        if (*pc == '.') {
          /* "x/.." to "x/../.." */
          netwib_er(netwib_buf_append_text("/..", pout));
          return(NETWIB_ERR_OK);
        }
      }
    }
  }

  pc = data + datasize - 1;
  while (NETWIB_TRUE) {
    if (*pc == '/') {
      break;
    }
    pc--;
    if (pc < data) {
      /* no '/' found, so : "x" to ".." */
      netwib__buf_erase(pout);
      netwib_er(netwib_buf_append_text("..", pout));
      return(NETWIB_ERR_OK);
    }
  }
  if (pc == data) {
    /* "/x" to "/" */
    pout->endoffset = pout->beginoffset + 1;
    *paddaslash = NETWIB_FALSE;
  } else {
    /* "x/aa/bb" to "x/aa" */
    pout->endoffset = pout->beginoffset + pc - data;
  }

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
static netwib_err netwib_priv_path_canon(netwib_conststring pathname,
                                         netwib_buf *pout)
{
  netwib_char c;
  netwib_conststring pcbegin, pcwork;
  netwib_string bufdirstr;
  netwib_byte bufarray[512];
  netwib_buf bufdir;
  netwib_bool addaslash;
  netwib_err ret;

  /* forbids empty filenames */
  c = pathname[0];
  if (pathname[0] == '\0') {
    return(NETWIB_ERR_SPPATHNOTCLEANED);
  }

  /* set root */
  addaslash = NETWIB_TRUE;
  if (c == '/' || c == '\\') {
    netwib_er(netwib_buf_append_byte('/', pout));
    addaslash = NETWIB_FALSE;
  } else {
    netwib_er(netwib_buf_append_byte('.', pout));
  }

  /* main loop treating each directory */
  netwib_er(netwib_buf_init_ext_arrayempty(bufarray, sizeof(bufarray),
                                           &bufdir));
  bufdir.flags |= NETWIB_BUF_FLAGS_CANALLOC;
  pcbegin = pathname;
  while (NETWIB_TRUE) {
    /* ensures pcbegin points to the next begin of directory */
    while (NETWIB_TRUE) {
      c = *pcbegin;
      if (c != '/' && c != '\\') {
        break;
      }
      pcbegin++;
    }
    /* if pcbegin is at end, quit */
    if (*pcbegin == '\0') {
      break;
    }
    /* obtain the next directory */
    pcwork = pcbegin;
    while (NETWIB_TRUE) {
      c = *pcwork;
      if (c == '/' || c == '\\' || c == '\0') {
        break;
      }
      pcwork++;
    }
    /* store it in bufdir */
    netwib__buf_reinit(&bufdir);
    netwib_er(netwib_buf_append_data((netwib_constdata)pcbegin,
                                     pcwork - pcbegin,
                                     &bufdir));
    pcbegin = pcwork;
    /* add depends on contents of bufdir */
    netwib_er(netwib_buf_ref_string(&bufdir, &bufdirstr));
    if (bufdirstr[0] == '.' && bufdirstr[1] == '\0') {
      /* add nothing */
    } else if (bufdirstr[0] == '.' && bufdirstr[1] == '.' &&
               bufdirstr[2] == '\0') {
      /* go up */
      ret = netwib_priv_path_canon_goup(pout, &addaslash);
      if (ret != NETWIB_ERR_OK) {
        netwib_er(netwib_buf_close(&bufdir));
        return(ret);
      }
    } else {
      /* add directory */
      if (addaslash) {
        netwib_er(netwib_buf_append_byte('/', pout));
      } else {
        /* now, the initial slash case is skiped */
        addaslash = NETWIB_TRUE;
      }
      netwib_er(netwib_buf_append_buf(&bufdir, pout));
    }
  }

  netwib_er(netwib_buf_close(&bufdir));
  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_path_canon(netwib_constbuf *pathname,
                             netwib_buf *pcanonizedpathname)
{
  netwib_string pc;
  netwib_conststring pcstartanalyze;
  netwib_byte bufstartarray[128], bufendcanonarray[512];
  netwib_buf bufstart, bufendcanon;
  netwib_char c1, c2, c3;
  netwib_err ret;

  /* obtain a pointer to string */
  netwib__buf_ref_string(pathname, pc, bufstorage,
                         netwib_path_canon(&bufstorage, pcanonizedpathname));

  /* pathname is empty */
  if (pc[0] == '\0') {
    return(NETWIB_ERR_SPPATHNOTCLEANED);
  }

  /* first, we find the start */
  c1 = *pc;
  c2 = *(pc + 1);               /* at the limit, it gets '\0' */
  if (c1 == '.') {
    netwib_er(netwib_buf_init_ext_text(".", &bufstart));
    pcstartanalyze = pc;
  } else if (c1 == '/' || (c1 == '\\' && c2 != '\\')) {
    netwib_er(netwib_buf_init_ext_text("/", &bufstart));
    pcstartanalyze = pc;
  } else if (c1 == '\\' && c2 == '\\') {
    /* search "s\sh" */
    netwib_er(netwib_buf_init_ext_arrayempty(bufstartarray,
                                             sizeof(bufstartarray),
                                             &bufstart));
    bufstart.flags |= NETWIB_BUF_FLAGS_CANALLOC;
    netwib_er(netwib_buf_append_text("\\\\", &bufstart));
    ret = netwib_priv_path_canon_searchsmb(pc + 2, &bufstart, &pcstartanalyze);
    if (ret == NETWIB_ERR_SPPATHNOTCLEANED) {
      netwib__buf_reinit(&bufstart);
      netwib_er(netwib_buf_append_byte('/', &bufstart));
      pcstartanalyze = pc;
    } else if (ret != NETWIB_ERR_OK) {
      netwib_er(netwib_buf_close(&bufstart));
      return(ret);
    }
  } else if (netwib_c2_isalpha(c1) && c2 == ':') {
    netwib_er(netwib_buf_init_ext_arrayempty(bufstartarray,
                                             sizeof(bufstartarray),
                                             &bufstart));
    bufstart.flags |= NETWIB_BUF_FLAGS_CANALLOC;
    netwib_c2_lower(c1);
    netwib_er(netwib_buf_append_byte(c1, &bufstart));
    c3 = *(pc + 2);             /* at the limit, it gets '\0' */
    if (c3 == '/' || c3 == '\\') {
      netwib_er(netwib_buf_append_text(":/", &bufstart));
    } else {
      netwib_er(netwib_buf_append_text(":.", &bufstart));
    }
    pcstartanalyze = pc + 2;
  } else {
    netwib_er(netwib_buf_init_ext_text(".", &bufstart));
    pcstartanalyze = pc;
  }

  /* if pcstartanalyze is at end, then there is only the start */
  if (*pcstartanalyze == '\0') {
    netwib_er(netwib_buf_append_buf(&bufstart, pcanonizedpathname));
    netwib_er(netwib_buf_close(&bufstart));
    return(NETWIB_ERR_OK);
  }

  /* now, we clean the path */
  netwib_er(netwib_buf_init_ext_arrayempty(bufendcanonarray,
                                           sizeof(bufendcanonarray),
                                           &bufendcanon));
  bufendcanon.flags |= NETWIB_BUF_FLAGS_CANALLOC;
  ret = netwib_priv_path_canon(pcstartanalyze, &bufendcanon);
  if (ret != NETWIB_ERR_OK) {
    netwib_er(netwib_buf_close(&bufstart));
    netwib_er(netwib_buf_close(&bufendcanon));
    return(ret);
  }

  /* now, we reconstruct the path */
  netwib_er(netwib_buf_append_buf(&bufstart, pcanonizedpathname));
  netwib_er(netwib_buf_close(&bufstart));
  bufendcanon.beginoffset++;
  netwib_er(netwib_buf_append_buf(&bufendcanon, pcanonizedpathname));
  netwib_er(netwib_buf_close(&bufendcanon));
  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_path_check(netwib_constbuf *pathname,
                             netwib_path_checktype type,
                             netwib_bool *pyes)
{
  netwib_byte buftmparray[512];
  netwib_buf buftmp;
  netwib_string strtmp;
  netwib_data data;
  netwib_bool yes;
  netwib_err ret;

  yes = NETWIB_FALSE;
  switch(type) {
    case NETWIB_PATH_CHECKTYPE_ABSOLUTE :
      netwib_er(netwib_buf_init_ext_arrayempty(buftmparray,
                                               sizeof(buftmparray), &buftmp));
      buftmp.flags |= NETWIB_BUF_FLAGS_CANALLOC;
      ret = netwib_path_decode_end(pathname, &buftmp);
      if (ret != NETWIB_ERR_OK) {
        netwib_er(netwib_buf_close(&buftmp));
        return(ret);
      }
      netwib_er(netwib_buf_ref_string(&buftmp, &strtmp));
      if (strtmp[0] == '/') yes = NETWIB_TRUE;
      netwib_er(netwib_buf_close(&buftmp));
      break;
    case NETWIB_PATH_CHECKTYPE_RELATIVE :
      if (netwib__buf_ref_data_size(pathname)) {
        data = netwib__buf_ref_data_ptr(pathname);
        if (data[0] == '.') yes = NETWIB_TRUE;
      }
      break;
    case NETWIB_PATH_CHECKTYPE_ROOT :
      netwib_er(netwib_buf_init_ext_arrayempty(buftmparray,
                                               sizeof(buftmparray), &buftmp));
      buftmp.flags |= NETWIB_BUF_FLAGS_CANALLOC;
      ret = netwib_path_decode_end(pathname, &buftmp);
      if (ret != NETWIB_ERR_OK) {
        netwib_er(netwib_buf_close(&buftmp));
        return(ret);
      }
      netwib_er(netwib_buf_ref_string(&buftmp, &strtmp));
      if (strtmp[0] == '/' && strtmp[1] == '\0') yes = NETWIB_TRUE;
      netwib_er(netwib_buf_close(&buftmp));
      break;
    case NETWIB_PATH_CHECKTYPE_START :
      netwib_er(netwib_buf_init_ext_arrayempty(buftmparray,
                                               sizeof(buftmparray), &buftmp));
      buftmp.flags |= NETWIB_BUF_FLAGS_CANALLOC;
      ret = netwib_path_decode_end(pathname, &buftmp);
      if (ret != NETWIB_ERR_OK) {
        netwib_er(netwib_buf_close(&buftmp));
        return(ret);
      }
      netwib_er(netwib_buf_ref_string(&buftmp, &strtmp));
      if ((strtmp[0] == '/' || strtmp[0] == '.') && strtmp[1] == '\0') {
        yes = NETWIB_TRUE;
      }
      netwib_er(netwib_buf_close(&buftmp));
      break;
    default :
      return(NETWIB_ERR_PAINVALIDTYPE);
      break;
  }

  if (pyes != NULL) {
    *pyes = yes;
  }
  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
static netwib_err netwib_priv_path_decode_startend(netwib_constbuf *pathname,
                                                   netwib_buf *pbufstart,
                                                   netwib_buf *pbufend)
{
  netwib_string pc, pbs;
  netwib_char c1, c2;

  netwib__buf_ref_string(pathname, pc, bufstorage,
                         netwib_priv_path_decode_startend(&bufstorage,
                                                          pbufstart, pbufend));

  /* first, we find the start */
  c1 = pc[0];
  c2 = pc[1];             /* at the limit, it gets '\0' */
  if (c1 == '.') {
    netwib_er(netwib_buf_append_byte('.', pbufstart));
    netwib_er(netwib_buf_append_buf(pathname, pbufend));
  } else if (c1 == '/') {
    netwib_er(netwib_buf_append_byte('/', pbufstart));
    netwib_er(netwib_buf_append_buf(pathname, pbufend));
  } else if (c1 == '\\' && c2 == '\\') {
    /* search "server\share\" */
    pbs = strchr(pc + 2, '\\');
    if (pbs == NULL)
      return(NETWIB_ERR_SPPATHNOTCLEANED);
    pbs = strchr(pbs + 1, '\\');
    if (pbs == NULL)
      return(NETWIB_ERR_SPPATHNOTCLEANED);
    netwib_er(netwib_buf_append_data((netwib_constdata)pc, pbs - pc + 1,
                                   pbufstart));
    netwib_er(netwib_buf_append_byte('/', pbufend));
    netwib_er(netwib_buf_append_text(pbs + 1, pbufend));
  } else if (netwib_c2_isalpha(c1) && c2 == ':') {
    netwib_er(netwib_buf_append_data((netwib_constdata)pc, 3, pbufstart));
    netwib_er(netwib_buf_append_text(pc + 2, pbufend));
  } else {
    netwib_er(netwib_buf_append_byte('.', pbufstart));
    netwib_er(netwib_buf_append_buf(pathname, pbufend));
  }

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_path_decode(netwib_constbuf *pathname,
                              netwib_path_decodetype type,
                              netwib_buf *pout)
{
  netwib_byte buftmparray[512];
  netwib_buf buftmp;
  netwib_string strtmp, pc;
  netwib_err ret;

  switch(type) {
    case NETWIB_PATH_DECODETYPE_ROOT :
      netwib_er(netwib_buf_init_ext_arrayempty(buftmparray,
                                               sizeof(buftmparray), &buftmp));
      buftmp.flags |= NETWIB_BUF_FLAGS_CANALLOC;
      netwib_er(netwib_priv_path_decode_startend(pathname, &buftmp, NULL));
      netwib_er(netwib_buf_ref_string(&buftmp, &strtmp));
      if (strtmp[netwib_c_strlen(strtmp) - 1] == '.') {
        netwib_er(netwib_buf_close(&buftmp));
        return(NETWIB_ERR_SPPATHRELATIVE);
      }
      netwib_er(netwib_buf_append_buf(&buftmp, pout));
      netwib_er(netwib_buf_close(&buftmp));
      break;
    case NETWIB_PATH_DECODETYPE_START :
      netwib_er(netwib_priv_path_decode_startend(pathname, pout, NULL));
      break;
    case NETWIB_PATH_DECODETYPE_END :
      netwib_er(netwib_priv_path_decode_startend(pathname, NULL, pout));
      break;
    case NETWIB_PATH_DECODETYPE_PARENTDIR :
      netwib_er(netwib_buf_init_ext_text("..", &buftmp));
      ret = netwib_path_init_concat(pathname, &buftmp, pout);
      return(ret);
      break;
    case NETWIB_PATH_DECODETYPE_LASTITEM :
      netwib_er(netwib_buf_init_ext_arrayempty(buftmparray,
                                               sizeof(buftmparray), &buftmp));
      buftmp.flags |= NETWIB_BUF_FLAGS_CANALLOC;
      netwib_er(netwib_priv_path_decode_startend(pathname, NULL, &buftmp));
      netwib_er(netwib_buf_ref_string(&buftmp, &strtmp));
      if ((strtmp[0] == '.' || strtmp[0] == '/') && strtmp[1] == '\0') {
        /* is "." or "/" */
        netwib_er(netwib_buf_append_buf(&buftmp, pout));
        netwib_er(netwib_buf_close(&buftmp));
        return(NETWIB_ERR_OK);
      }
      /* search the last '/' */
      pc = strrchr(strtmp, '/');
      if (pc == NULL) {
        netwib_er(netwib_buf_close(&buftmp));
        netwib_er(netwib_priv_errmsg_func_text("netwib_path_decode", ""));
        return(NETWIB_ERR_IEINTERNALERROR);
      }
      netwib_er(netwib_buf_append_text(pc + 1, pout));
      netwib_er(netwib_buf_close(&buftmp));
      break;
    default :
      return(NETWIB_ERR_PAINVALIDTYPE);
      break;
 }

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
static netwib_err netwib_priv_path_init_concat(netwib_constbuf *dirname1,
                                               netwib_constbuf *pathname2,
                                               netwib_buf *pout)
{
  netwib_byte buftmparray[512];
  netwib_buf buftmp;
  netwib_bool b;
  netwib_char c;
  netwib_err ret;

  /* there is not limit on dirname1
     pathname2 must start with . or /
     if pathname2 starts with /, dirname1 must be a root
   */
  c = '_';
  if (netwib__buf_ref_data_size(pathname2)) {
    c = *((netwib_char*)netwib__buf_ref_data_ptr(pathname2));
  }
  if (c == '.') {
    /* always ok */
  } else if (c == '/') {
    netwib_er(netwib_path_check_root(dirname1, &b));
    if (!b)
      return(NETWIB_ERR_SPPATHCANTINIT);
  } else {
    return(NETWIB_ERR_SPPATHCANTINIT);
  }

  netwib_er(netwib_buf_init_ext_arrayempty(buftmparray, sizeof(buftmparray),
                                           &buftmp));
  buftmp.flags |= NETWIB_BUF_FLAGS_CANALLOC;
  netwib_er(netwib_buf_append_buf(dirname1, &buftmp));
  netwib_er(netwib_buf_append_byte('/', &buftmp));
  netwib_er(netwib_buf_append_buf(pathname2, &buftmp));
  ret = netwib_path_canon(&buftmp, pout);
  netwib_er(netwib_buf_close(&buftmp));

  return(ret);
}

/*-------------------------------------------------------------*/
static netwib_err netwib_priv_path_init_abs(netwib_constbuf *dirname1,
                                            netwib_constbuf *pathname2,
                                            netwib_buf *pout)
{
  netwib_bool b;

  netwib_er(netwib_path_check_absolute(pathname2, &b));
  if (b) {
    netwib_er(netwib_buf_append_buf(pathname2, pout));
    return(NETWIB_ERR_OK);
  }

  netwib_er(netwib_path_check_absolute(dirname1, &b));
  if (!b) {
    return(NETWIB_ERR_SPPATHCANTINIT);
  }

  netwib_er(netwib_priv_path_init_concat(dirname1, pathname2, pout));

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
static netwib_err netwib_priv_path_init_rel(netwib_constbuf *dirname1,
                                            netwib_constbuf *pathname2,
                                            netwib_buf *pout)
{
  netwib_string pc1, pc1lastslash, pc1startcount, pc2, pc2lastslash;
  netwib_byte buftmparray[512];
  netwib_buf buftmp;
  netwib_err ret;

  netwib__buf_ref_string(dirname1, pc1, bufstorage,
                         netwib_priv_path_init_rel(&bufstorage, pathname2,
                                                   pout));
  netwib__buf_ref_string(pathname2, pc2, bufstorage,
                         netwib_priv_path_init_rel(dirname1, &bufstorage,
                                                   pout));

  /* find the common part */
  pc1lastslash = pc1;       /* init for relative cases (1="./" and 2="../") */
  pc1startcount = NULL;
  pc2lastslash = pc2;       /* init for relative cases (1="./" and 2="../") */
  while (NETWIB_TRUE) {
    if (*pc1 == '\0') {
      if (*pc2 == '/') {
        pc1startcount = pc1;
        pc2lastslash = pc2;
      } else if (*pc2 == '\0') {
        pc1startcount = pc1lastslash;
      } else if (*(pc1 - 1) == '/') {
        pc1startcount = pc1;
      } else {
        pc1startcount = pc1lastslash;
      }
      break;
    }
    if (*pc2 == '\0') {
      pc1startcount = pc1lastslash;
      break;
    }
    if (*pc1 != *pc2) {
      pc1startcount = pc1lastslash;
      break;
    }
    if (*pc1 == '/') {
      pc1lastslash = pc1;
    }
    if (*pc2 == '/') {
      pc2lastslash = pc2;
    }
    pc1++;
    pc2++;
  }

  /* if pc1startcount points to "../xx", we cannot determine */
  if (netwib_c_strlen(pc1startcount) > 2) {
    if (*(pc1startcount) == '.' &&
        *(pc1startcount + 1) == '.' && *(pc1startcount + 2) == '/') {
      return(NETWIB_ERR_SPPATHCANTINIT);
    }
  }

  /* now loop till the end of pc1 to create "../../../" */
  netwib_er(netwib_buf_init_ext_arrayempty(buftmparray, sizeof(buftmparray),
                                           &buftmp));
  buftmp.flags |= NETWIB_BUF_FLAGS_CANALLOC;
  netwib_er(netwib_buf_append_text("./", &buftmp));
  pc1 = pc1startcount;
  while (*pc1 != '\0') {
    if (*pc1 == '/') {
      netwib_er(netwib_buf_append_text("../", &buftmp));
    }
    pc1++;
  }

  /* now concatene */
  netwib_er(netwib_buf_append_text(pc2lastslash, &buftmp));
  ret = netwib_path_canon(&buftmp, pout);
  netwib_er(netwib_buf_close(&buftmp));
  return(ret);
}

/*-------------------------------------------------------------*/
static netwib_err netwib_priv_path_init_rela(netwib_constbuf *dirname1,
                                             netwib_constbuf *pathname2,
                                             netwib_buf *pout)
{
  netwib_byte buf1array[512], buf2array[512];
  netwib_buf buf1, buf2;
  netwib_string pc1, pc2;
  netwib_bool b;
  netwib_err ret;

  /* simple case */
  netwib_er(netwib_path_check_relative(pathname2, &b));
  if (b) {
    netwib_er(netwib_buf_append_buf(pathname2, pout));
    return(NETWIB_ERR_OK);
  }

  /* now we have to ensure starts are equal to continue */
  netwib_er(netwib_buf_init_ext_arrayempty(buf1array, sizeof(buf1array),
                                           &buf1));
  netwib_er(netwib_buf_init_ext_arrayempty(buf2array, sizeof(buf2array),
                                           &buf2));
  buf1.flags |= NETWIB_BUF_FLAGS_CANALLOC;
  buf2.flags |= NETWIB_BUF_FLAGS_CANALLOC;
  netwib_er(netwib_path_decode_start(dirname1, &buf1));
  netwib_er(netwib_path_decode_start(pathname2, &buf2));
  netwib_er(netwib_buf_ref_string(&buf1, &pc1));
  netwib_er(netwib_buf_ref_string(&buf2, &pc2));
  if (netwib_c_strcmp(pc1, pc2)) {
    netwib_er(netwib_buf_close(&buf1));
    netwib_er(netwib_buf_close(&buf2));
    return(NETWIB_ERR_SPPATHCANTINIT);
  }

  /* continue our work with ends only */
  netwib__buf_reinit(&buf1);
  netwib__buf_reinit(&buf2);
  netwib_er(netwib_path_decode_end(dirname1, &buf1));
  netwib_er(netwib_path_decode_end(pathname2, &buf2));

  /* simple case */
  netwib_er(netwib_path_check_relative(&buf2, &b));
  if (b) {
    netwib_er(netwib_buf_append_buf(&buf2, pout));
    netwib_er(netwib_buf_close(&buf1));
    netwib_er(netwib_buf_close(&buf2));
    return(NETWIB_ERR_OK);
  }

  /* we need both to be absolute */
  netwib_er(netwib_path_check_absolute(&buf1, &b));
  if (!b) {
    netwib_er(netwib_buf_close(&buf1));
    netwib_er(netwib_buf_close(&buf2));
    return(NETWIB_ERR_SPPATHCANTINIT);
  }

  /* now, compute the relative path */
  ret = netwib_priv_path_init_rel(&buf1, &buf2, pout);
  netwib_er(netwib_buf_close(&buf1));
  netwib_er(netwib_buf_close(&buf2));
  return(ret);
}

/*-------------------------------------------------------------*/
static netwib_err netwib_priv_path_init_rela2(netwib_constbuf *dircommon,
                                              netwib_constbuf *dirname1,
                                              netwib_constbuf *pathname2,
                                              netwib_buf *pout)
{
  netwib_bool b;
  netwib_byte np1array[512];
  netwib_buf np1;
  netwib_err ret;

  netwib_er(netwib_path_check_relative(dirname1, &b));
  if (b) {
    netwib_er(netwib_buf_init_ext_arrayempty(np1array, sizeof(np1array),
                                             &np1));
    np1.flags |= NETWIB_BUF_FLAGS_CANALLOC;
    netwib_er(netwib_path_init_concat(dircommon, dirname1, &np1));
    ret = netwib_priv_path_init_rela(&np1, pathname2, pout);
    netwib_er(netwib_buf_close(&np1));
  } else {
    ret = netwib_priv_path_init_rela(dirname1, pathname2, pout);
  }
  return(ret);
}

/*-------------------------------------------------------------*/
static netwib_err netwib_priv_path_init_relb(netwib_constbuf *dirname1,
                                             netwib_constbuf *pathname2,
                                             netwib_buf *pout)
{
  netwib_byte buf1array[512], buf2array[512];
  netwib_buf buf1, buf2;
  netwib_string pc1, pc2;
  netwib_bool b1, b2;
  netwib_err ret;

  /* we have to ensure starts are equal to continue */
  netwib_er(netwib_buf_init_ext_arrayempty(buf1array, sizeof(buf1array),
                                           &buf1));
  netwib_er(netwib_buf_init_ext_arrayempty(buf2array, sizeof(buf2array),
                                           &buf2));
  buf1.flags |= NETWIB_BUF_FLAGS_CANALLOC;
  buf2.flags |= NETWIB_BUF_FLAGS_CANALLOC;
  netwib_er(netwib_path_decode_start(dirname1, &buf1));
  netwib_er(netwib_path_decode_start(pathname2, &buf2));
  netwib_er(netwib_buf_ref_string(&buf1, &pc1));
  netwib_er(netwib_buf_ref_string(&buf2, &pc2));
  if (netwib_c_strcmp(pc1, pc2)) {
    netwib_er(netwib_buf_close(&buf1));
    netwib_er(netwib_buf_close(&buf2));
    return(NETWIB_ERR_SPPATHCANTINIT);
  }

  /* continue our work with ends only */
  netwib__buf_reinit(&buf1);
  netwib__buf_reinit(&buf2);
  netwib_er(netwib_path_decode_end(dirname1, &buf1));
  netwib_er(netwib_path_decode_end(pathname2, &buf2));

  /* we need both to be relative or absolute */
  netwib_er(netwib_path_check_relative(&buf1, &b1));
  netwib_er(netwib_path_check_relative(&buf2, &b2));
  if ((b1 && !b2) || (!b1 && b2)) {
    netwib_er(netwib_buf_close(&buf1));
    netwib_er(netwib_buf_close(&buf2));
    return(NETWIB_ERR_SPPATHCANTINIT);
  }

  /* now, compute the relative path */
  ret = netwib_priv_path_init_rel(&buf1, &buf2, pout);
  netwib_er(netwib_buf_close(&buf1));
  netwib_er(netwib_buf_close(&buf2));
  return(ret);
}

/*-------------------------------------------------------------*/
static netwib_err netwib_priv_path_init_relb2(netwib_constbuf *dircommon,
                                              netwib_constbuf *dirname1,
                                              netwib_constbuf *pathname2,
                                              netwib_buf *pout)
{
  netwib_bool b1, b2;
  netwib_byte np1array[512], np2array[512];
  netwib_buf np1, np2;
  netwib_err ret;

  /* use pathcommon if paths are relative */
  netwib_er(netwib_path_check_relative(dirname1, &b1));
  netwib_er(netwib_path_check_relative(pathname2, &b2));

  netwib_er(netwib_buf_init_ext_arrayempty(np1array, sizeof(np1array), &np1));
  netwib_er(netwib_buf_init_ext_arrayempty(np2array, sizeof(np2array), &np2));
  np1.flags |= NETWIB_BUF_FLAGS_CANALLOC;
  np2.flags |= NETWIB_BUF_FLAGS_CANALLOC;

  if (b1 && b2) {
    ret = netwib_path_init_concat(dircommon, dirname1, &np1);
    if (ret == NETWIB_ERR_OK) {
      ret = netwib_path_init_concat(dircommon, pathname2, &np2);
      if (ret == NETWIB_ERR_OK) {
        ret = netwib_priv_path_init_relb(&np1, &np2, pout);
      }
    }
  } else if (b1) {
    ret = netwib_path_init_concat(dircommon, dirname1, &np1);
    if (ret == NETWIB_ERR_OK) {
      ret = netwib_priv_path_init_relb(&np1, pathname2, pout);
    }
  } else if (b2) {
    ret = netwib_path_init_concat(dircommon, pathname2, &np2);
    if (ret == NETWIB_ERR_OK) {
      ret = netwib_priv_path_init_relb(dirname1, &np2, pout);
    }
  } else {
    ret = netwib_priv_path_init_relb(dirname1, pathname2, pout);
  }

  netwib_er(netwib_buf_close(&np1));
  netwib_er(netwib_buf_close(&np2));

  return(ret);
}

/*-------------------------------------------------------------*/
netwib_err netwib_path_init(netwib_constbuf *dircommon,
                            netwib_constbuf *dirname1,
                            netwib_constbuf *pathname2,
                            netwib_path_inittype type,
                            netwib_buf *pout)
{
  switch(type) {
    case NETWIB_PATH_INITTYPE_CONCAT :
      return(netwib_priv_path_init_concat(dirname1, pathname2, pout));
      break;
    case NETWIB_PATH_INITTYPE_ABS :
      return(netwib_priv_path_init_abs(dirname1, pathname2, pout));
      break;
    case NETWIB_PATH_INITTYPE_RELA :
      return(netwib_priv_path_init_rela(dirname1, pathname2, pout));
      break;
    case NETWIB_PATH_INITTYPE_RELA2 :
      return(netwib_priv_path_init_rela2(dircommon, dirname1, pathname2,
                                         pout));
      break;
    case NETWIB_PATH_INITTYPE_RELB :
      return(netwib_priv_path_init_relb(dirname1, pathname2, pout));
      break;
    case NETWIB_PATH_INITTYPE_RELB2 :
      return(netwib_priv_path_init_relb2(dircommon, dirname1, pathname2,
                                         pout));
      break;
  }

  return(NETWIB_ERR_OK);
}
