/* This file is part of Netwib.
*/

/*-------------------------------------------------------------*/
netwib_err netwib_priv_err_purge(void)
{
  netwib_er(netwib_priv_errmsg_reinit());

  errno = 0;
#if defined NETWIBDEF_SYSNAME_Unix
 #if NETWIBDEF_HAVEVAR_HERRNO==1
  h_errno = 0;
 #endif
#elif defined NETWIBDEF_SYSNAME_Windows
  WSASetLastError(0);
  SetLastError(0);
#else
#error "Unknown value for NETWIBDEF_SYSNAME"
#endif

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
static netwib_err netwib_priv_err_text(netwib_err error,
                                       netwib_conststring *pc,
                                       netwib_bool *padderrorin)
{

  *pc = NULL;
  switch(error) {
    /**/
    case NETWIB_ERR_OK:
      *pc = "ok";
      break;
    case NETWIB_ERR_OKDATAEND:
      *pc = "definitive end of data";
      break;
    case NETWIB_ERR_OKDATANOTAVAIL:
      *pc = "there is no data available";
      break;
    case NETWIB_ERR_OKDATANOSPACE:
      *pc = "no sufficient space to store data";
      break;
    case NETWIB_ERR_OKDATAOTHERTYPE:
      *pc = "data is present but does not have the expected type";
      break;
    case NETWIB_ERR_OKNOTFOUND:
      *pc = "not found";
      break;
    case NETWIB_ERR_OKNOTDECODED:
      *pc = "not decoded";
      break;
    case NETWIB_ERR_OKNOTCONVERTED:
      *pc = "not converted";
      break;
    case NETWIB_ERR_OKROUTELOCALDEV:
      *pc = "the destination is a local device";
      break;
    case NETWIB_ERR_OKROUTENOTFOUND:
      *pc = "no route found to destination";
      break;
    case NETWIB_ERR_OKROUTENOTETHER:
      *pc = "Ethernet is not used to reach destination";
      break;
    case NETWIB__ERR_OK_END:
      *pc = "end of OK error family";
      break;

    /**/
    case NETWIB_ERR_FM:
      *pc = "format error family";
      break;
    case NETWIB_ERR_FMHEXACHAR:
      *pc = "an hexa string must not contain a character different from A-Fa-f0-9";
      break;
    case NETWIB_ERR_FMHEXAODD:
      *pc = "the byte number in the hexa string is odd";
      break;
    case NETWIB_ERR_FMMIXED:
      *pc = "the mixed string doesn't finish by '";
      break;
    case NETWIB_ERR_FMBASE64CHAR:
      *pc = "all characters in a base64 must be in A-Za-z0-9+/=";
      break;
    case NETWIB_ERR_FMBASE64X4:
      *pc = "with base64, the byte number must be a multiple of 4";
      break;
    case NETWIB_ERR_FMBASE64LO:
      *pc = "base64 string is badly structured";
      break;
    case NETWIB_ERR_FMFMT:
      *pc = "the format is incorrect";
      break;
    case NETWIB_ERR_FMINT:
      *pc = "the string is not an integer";
      break;
    case NETWIB__ERR_FM_END:
      *pc = "end of FM error family";
      break;

    /**/
    case NETWIB_ERR_PA:
      *pc = "parameter error family";
      break;
    case NETWIB_ERR_PAINVALIDTYPE:
      *pc = "invalid type";
      break;
    case NETWIB_ERR_PAINVALIDPOS:
      *pc = "invalid position";
      break;
    case NETWIB_ERR_PAINVALIDRANGE:
      *pc = "invalid range";
      break;
    case NETWIB_ERR_PAINVALIDDEFAULT:
      *pc = "invalid default value";
      break;
    case NETWIB_ERR_PANULLPTR:
      *pc = "a NULL pointer is not allowed";
      break;
    case NETWIB_ERR_PANULLPTRSIZE:
      *pc = "pointer is NULL, but size isn't zero";
      break;
    case NETWIB_ERR_PATOOLOW:
      *pc = "parameter is too low";
      break;
    case NETWIB_ERR_PATOOHIGH:
      *pc = "parameter is too high";
      break;
    case NETWIB__ERR_PA_END:
      *pc = "end of PA error family";
      break;

    /**/
    case NETWIB_ERR_SP:
      *pc = "specific parameter error family";
      break;
    case NETWIB_ERR_SPUNIXSIGHDLBADSIG:
      *pc = "signal is not SIGINT or SIGTERM";
      break;
    case NETWIB_ERR_SPRINGFULL:
      *pc = "the ring cannot have more than 0x7FFFFFFF elements";
      break;
    case NETWIB_ERR_SPTLVINVALID:
      *pc = "invalid TLV";
      break;
    case NETWIB_ERR_SPTIMEDIFFNEG:
      *pc = "time difference is negative";
      break;
    case NETWIB_ERR_SPPATHROOTDOTDOT:
      *pc = "too many .. in the path : go up /";
      break;
    case NETWIB_ERR_SPPATHNOTCLEANED:
      *pc = "the path could not be cleaned";
      break;
    case NETWIB_ERR_SPPATHRELATIVE:
      *pc = "the path is relative";
      break;
    case NETWIB_ERR_SPPATHCANTINIT:
      *pc = "it's not possible to init a path from the two given pathnames";
      break;
    case NETWIB_ERR_SPFILE2G:
      *pc = "file size greater than 2Gbytes";
      break;
    case NETWIB_ERR_SPFILECHANGED:
      *pc = "file changed";
      break;
    case NETWIB_ERR_SPFILESYMLINK:
      *pc = "file is a symlink";
      break;
    case NETWIB_ERR_SPFILENOTREG:
      *pc = "file is not regular";
      break;
    case NETWIB_ERR_SPINDEXNODATA:
      *pc = "index does not point to data (i.e. is just rewinded)";
      break;
    case NETWIB_ERR_SPTOOBIGFORHDR:
      *pc = "a parameter is too big to go in the physical header";
      break;
    case NETWIB_ERR_SPIP4OPTSNOTX4:
      *pc = "the ip options size is not a multiple of 4";
      break;
    case NETWIB_ERR_SPIP4OPTSMAX10:
      *pc = "you cannot have more than 10 lines of ip options";
      break;
    case NETWIB_ERR_SPIP6EXTSNOTX4:
      *pc = "the ip extensions size is not a multiple of 4";
      break;
    case NETWIB_ERR_SPTCPOPTSNOTX4:
      *pc = "the tcp options size is not a multiple of 4";
      break;
    case NETWIB_ERR_SPTCPOPTSMAX10:
      *pc = "you cannot have more than 10 lines of tcp options";
      break;
    case NETWIB_ERR_SPIPTYPE:
      *pc = "bad IP type";
      break;
    case NETWIB_ERR_SPIPTYPENOT4:
      *pc = "IP type is not IPv4";
      break;
    case NETWIB_ERR_SPIPTYPENOT6:
      *pc = "IP type is not IPv6";
      break;
    case NETWIB__ERR_SP_END:
      *pc = "end of SP error family";
      break;

    /**/
    case NETWIB_ERR_LO:
      *pc = "logic error family";
      break;
    case NETWIB_ERR_LONOTIMPLEMENTED:
      *pc = "not implemented";
      break;
    case NETWIB_ERR_LONOTSUPPORTED:
      *pc = "not supported";
      break;
    case NETWIB_ERR_LOERRORUNKNOWN:
      *pc = "error unknown by netwib";
      break;
    case NETWIB_ERR_LOERROROKKO:
      *pc = "NETWIB_ERR_OK is returned by the function, but this should not happen";
      break;
    case NETWIB_ERR_LOOBJINITALREADYINIT:
      *pc = "already initialized";
      break;
    case NETWIB_ERR_LOOBJUSENOTINITIALIZED:
      *pc = "use of an object not initialized";
      break;
    case NETWIB_ERR_LOOBJUSECLOSED:
      *pc = "use of a closed object";
      break;
    case NETWIB_ERR_LOOBJUSECORRUPT:
      *pc = "use of a corrupted object";
      break;
    case NETWIB_ERR_LOOBJCLOSENOTINITIALIZED:
      *pc = "closing an object not initialized";
      break;
    case NETWIB_ERR_LOOBJCLOSEALREADYCLOSED:
      *pc = "already closed";
      break;
    case NETWIB_ERR_LOOBJCLOSECORRUPT:
      *pc = "closing a corrupted object";
      break;
    case NETWIB_ERR_LOOBJREADNOTSUPPORTED:
      *pc = "reading is not supported";
      break;
    case NETWIB_ERR_LOOBJREADNOTALLOWED:
      *pc = "reading is not allowed";
      break;
    case NETWIB_ERR_LOOBJWRITENOTSUPPORTED:
      *pc = "writing is not supported";
      break;
    case NETWIB_ERR_LOOBJWRITENOTALLOWED:
      *pc = "writing is not allowed";
      break;
    case NETWIB_ERR_LOOBJRDWRCONFLICT:
      *pc = "reading and writing return different values";
      break;
    case NETWIB_ERR_LOCANTEXEC:
      *pc = "cannot execute";
      break;
    case NETWIB_ERR_LOUDPSERREADBEFWRITE:
      *pc = "it is not possible to write before reading on a udp server";
      break;
    case NETWIB__ERR_LO_END:
      *pc = "end of LO error family";
      break;

    /**/
    case NETWIB_ERR_IE:
      *pc = "internal error family";
      break;
    case NETWIB_ERR_IEINTERNALERROR:
      *pc = "internal error : contact Laurent";
      break;
    case NETWIB__ERR_IE_END:
      *pc = "end of IE error family";
      break;

    /**/
    case NETWIB_ERR_PL:
      *pc = "please error family";
      break;
    case NETWIB_ERR_PLTRYNEXT:
      *pc = "please, try next io";
      break;
    case NETWIB_ERR_PLCONSTRUCT:
      *pc = "please, construct it";
      break;
    case NETWIB_ERR_PLLOOPTIME:
      *pc = "please, loop to reach time";
      break;
    case NETWIB__ERR_PL_END:
      *pc = "end of PL error family";
      break;

    /**/
    case NETWIB_ERR_FU:
      *pc = "system function error family";
      break;
    case NETWIB__ERR_FU_END:
      *pc = "end of FU error family";
      break;

    /* special */
    case NETWIB_ERR_FUACCEPT: *pc = "accept"; break;
    case NETWIB_ERR_FUBIND: *pc = "bind"; break;
    case NETWIB_ERR_FUCHSIZE: *pc = "chsize"; break;
    case NETWIB_ERR_FUCLOCKGETTIME: *pc = "chsize"; break;
    case NETWIB_ERR_FUCLOSEDIR: *pc = "closedir"; break;
    case NETWIB_ERR_FUCLOSEHANDLE: *pc = "CloseHandle"; break;
    case NETWIB_ERR_FUCLOSE: *pc = "close"; break;
    case NETWIB_ERR_FUCONNECT: *pc = "connect"; break;
    case NETWIB_ERR_FUCREATEEVENT: *pc = "CreateEvent"; break;
    case NETWIB_ERR_FUCREATEFILE: *pc = "CreateFile"; break;
    case NETWIB_ERR_FUCREATEMUTEX: *pc = "CreateMutex"; break;
    case NETWIB_ERR_FUCREATEPIPE: *pc = "CreatePipe"; break;
    case NETWIB_ERR_FUCREATEPROCESS: *pc = "CreateProcess"; break;
    case NETWIB_ERR_FUCREATETHREAD: *pc = "CreateThread"; break;
    case NETWIB_ERR_FUDEVICEIOCONTROL: *pc = "DeviceIoControl"; break;
    case NETWIB_ERR_FUDUP2: *pc = "dup2"; break;
    case NETWIB_ERR_FUDUPLICATEHANDLE: *pc = "DuplicateHandle"; break;
    case NETWIB_ERR_FUEXECLE: *pc = "execle"; break;
    case NETWIB_ERR_FUEXECVE: *pc = "execve"; break;
    case NETWIB_ERR_FUFCHMOD: *pc = "fchmod"; break;
    case NETWIB_ERR_FUFCHOWN: *pc = "fchown"; break;
    case NETWIB_ERR_FUFCLOSE: *pc = "fclose"; break;
    case NETWIB_ERR_FUFCNTL: *pc = "fcntl"; break;
    case NETWIB_ERR_FUFDOPEN: *pc = "fdopen"; break;
    case NETWIB_ERR_FUFFLUSH: *pc = "fflush"; break;
    case NETWIB_ERR_FUFGETS: *pc = "fgets"; break;
    case NETWIB_ERR_FUFILENO: *pc = "fileno"; break;
    case NETWIB_ERR_FUFINDCLOSE: *pc = "findclose"; break;
    case NETWIB_ERR_FUFINDFIRST: *pc = "findfirst"; break;
    case NETWIB_ERR_FUFOPEN: *pc = "fopen"; break;
    case NETWIB_ERR_FUFORK: *pc = "fork"; break;
    case NETWIB_ERR_FUFREELIBRARY: *pc = "FreeLibrary"; break;
    case NETWIB_ERR_FUFREAD: *pc = "fread"; break;
    case NETWIB_ERR_FUFSEEK: *pc = "fseek"; break;
    case NETWIB_ERR_FUFSTAT: *pc = "fstat"; break;
    case NETWIB_ERR_FUFSYNC: *pc = "fsync"; break;
    case NETWIB_ERR_FUFTRUNCATE: *pc = "ftruncate"; break;
    case NETWIB_ERR_FUFWRITE: *pc = "fwrite"; break;
    case NETWIB_ERR_FUGENERATECONSOLECTRLEVENT: *pc = "GenerateConsoleCtrlEvent"; break;
    case NETWIB_ERR_FUGETCONSOLEMODE: *pc = "GetConsoleMode"; break;
    case NETWIB_ERR_FUGETEXITCODEPROCESS: *pc = "GetExitCodeProcess"; break;
    case NETWIB_ERR_FUGETEXITCODETHREAD: *pc = "GetExitCodeThread"; break;
    case NETWIB_ERR_FUGETGRGIDR: *pc = "getgrgid_r"; break;
    case NETWIB_ERR_FUGETGRNAMR: *pc = "getgrnam_r"; break;
    case NETWIB_ERR_FUGETHOSTNAME: *pc = "gethostname"; break;
    case NETWIB_ERR_FUGETHOSTBYNAMER: *pc = "gethostbyname_r"; break;
    case NETWIB_ERR_FUGETHOSTBYADDRR: *pc = "gethostbyaddr_r"; break;
    case NETWIB_ERR_FUGETIFTABLE: *pc = "GetIfTable"; break;
    case NETWIB_ERR_FUGETIPADDRTABLE: *pc = "GetIpAddrTable"; break;
    case NETWIB_ERR_FUGETIPFORWARDTABLE: *pc = "GetIpForwardTable"; break;
    case NETWIB_ERR_FUGETIPNETTABLE: *pc = "GetIpNetTable"; break;
    case NETWIB_ERR_FUGETMSG: *pc = "getmsg"; break;
    case NETWIB_ERR_FUGETNUMBEROFINTERFACES: *pc = "GetNumberOfInterfaces"; break;
    case NETWIB_ERR_FUGETPEERNAME: *pc = "getpeername"; break;
    case NETWIB_ERR_FUGETPRIORITY: *pc = "getpriority"; break;
    case NETWIB_ERR_FUGETPROCADDRESS: *pc = "GetProcAddress"; break;
    case NETWIB_ERR_FUGETRLIMIT: *pc = "getrlimit"; break;
    case NETWIB_ERR_FUGETPWNAMR: *pc = "getpwnam_r"; break;
    case NETWIB_ERR_FUGETSOCKNAME: *pc = "getsockname"; break;
    case NETWIB_ERR_FUGETSTDHANDLE: *pc = "GetStdHandle"; break;
    case NETWIB_ERR_FUGETTHREADPRIORITY: *pc = "GetThreadPriority"; break;
    case NETWIB_ERR_FUGETTIMEOFDAY: *pc = "gettimeofday"; break;
    case NETWIB_ERR_FUGLOBALALLOC: *pc = "GlobalAlloc"; break;
    case NETWIB_ERR_FUGLOBALFREE: *pc = "GlobalFree"; break;
    case NETWIB_ERR_FULOCALTIMER: *pc = "localtime_r"; break;
    case NETWIB_ERR_FUIOCTL: *pc = "ioctl"; break;
    case NETWIB_ERR_FUKILL: *pc = "kill"; break;
    case NETWIB_ERR_FULIBNETINIT: *pc = "libnet_init"; break;
    case NETWIB_ERR_FULIBNETCLOSELINKINT: *pc = "libnet_close_link_interface"; break;
    case NETWIB_ERR_FULIBNETCLOSERAWSOCK: *pc = "libnet_close_raw_sock"; break;
    case NETWIB_ERR_FULIBNETGETHWADDR: *pc = "libnet_get_hwaddr"; break;
    case NETWIB_ERR_FULIBNETOPENLINKINT: *pc = "libnet_open_link_interface"; break;
    case NETWIB_ERR_FULIBNETOPENRAWSOCK: *pc = "libnet_open_raw_sock"; break;
    case NETWIB_ERR_FULIBNETWRITEIP: *pc = "libnet_write_ip"; break;
    case NETWIB_ERR_FULIBNETWRITELINK: *pc = "libnet_write_link"; break;
    case NETWIB_ERR_FULIBNETWRITELL: *pc = "libnet_write_link_layer"; break;
    case NETWIB_ERR_FULIBNETWRITERAWIPV4: *pc = "libnet_write_raw_ipv4"; break;
    case NETWIB_ERR_FULIBNETWRITERAWIPV6: *pc = "libnet_write_raw_ipv6"; break;
    case NETWIB_ERR_FULISTEN: *pc = "listen"; break;
    case NETWIB_ERR_FULOADLIBRARY: *pc = "LoadLibrary"; break;
    case NETWIB_ERR_FULOCALALLOC: *pc = "LocalAlloc"; break;
    case NETWIB_ERR_FULOCALFREE: *pc = "LocalFree"; break;
    case NETWIB_ERR_FULSEEK: *pc = "lseek"; break;
    case NETWIB_ERR_FULSTAT: *pc = "lstat"; break;
    case NETWIB_ERR_FUMALLOC: *pc = "malloc"; break;
    case NETWIB_ERR_FUMKDIR: *pc = "mkdir"; break;
    case NETWIB_ERR_FUMPROTECT: *pc = "mprotect"; break;
    case NETWIB_ERR_FUNANOSLEEP: *pc = "nanosleep"; break;
    case NETWIB_ERR_FUOPENDIR: *pc = "opendir"; break;
    case NETWIB_ERR_FUOPEN: *pc = "open"; break;
    case NETWIB_ERR_FUOPENPROCESS: *pc = "OpenProcess"; break;
    case NETWIB_ERR_FUPACKETALLOCATEPACKET: *pc = "PacketAllocatePacket"; break;
    case NETWIB_ERR_FUPACKETGETADAPTERNAMES: *pc = "PacketGetAdapterNames"; break;
    case NETWIB_ERR_FUPACKETGETNETTYPE: *pc = "PacketGetNetType"; break;
    case NETWIB_ERR_FUPACKETOPENADAPTER: *pc = "PacketOpenAdapter"; break;
    case NETWIB_ERR_FUPACKETREQUEST: *pc = "PacketRequest"; break;
    case NETWIB_ERR_FUPACKETSENDPACKET: *pc = "PacketSendPacket"; break;
    case NETWIB_ERR_FUPACKETSETBUFF: *pc = "PacketSetBuff"; break;
    case NETWIB_ERR_FUPCAPCOMPILE: *pc = "pcap_compile"; break;
    case NETWIB_ERR_FUPCAPDUMPOPEN: *pc = "pcap_dump_open"; break;
    case NETWIB_ERR_FUPCAPFINDALLDEVS: *pc = "pcap_findalldevs"; break;
    case NETWIB_ERR_FUPCAPGETEVENT: *pc = "pcap_getevent"; break;
    case NETWIB_ERR_FUPCAPLOOKUPDEV: *pc = "pcap_lookupdev"; break;
    case NETWIB_ERR_FUPCAPLOOKUPNET: *pc = "pcap_lookupnet"; break;
    case NETWIB_ERR_FUPCAPOPENDEAD: *pc = "pcap_open_dead"; break;
    case NETWIB_ERR_FUPCAPOPENOFFLINE: *pc = "pcap_open_offline"; break;
    case NETWIB_ERR_FUPCAPOPENLIVE: *pc = "pcap_open_live"; break;
    case NETWIB_ERR_FUPCAPSETFILTER: *pc = "pcap_setfilter"; break;
    case NETWIB_ERR_FUPCLOSE: *pc = "pclose"; break;
    case NETWIB_ERR_FUPEEKCONSOLEINPUT: *pc = "PeekConsoleInput"; break;
    case NETWIB_ERR_FUPEEKNAMEDPIPE: *pc = "PeekNamedPipe"; break;
    case NETWIB_ERR_FUPIPE: *pc = "pipe"; break;
    case NETWIB_ERR_FUPOLL: *pc = "poll"; break;
    case NETWIB_ERR_FUPOPEN: *pc = "popen"; break;
    case NETWIB_ERR_FUPTHREADATTRINIT: *pc = "pthread_attr_init"; break;
    case NETWIB_ERR_FUPTHREADATTRDESTROY: *pc = "pthread_attr_destroy"; break;
    case NETWIB_ERR_FUPTHREADATTRSETATTR: *pc = "pthread_attr_setattr"; break;
    case NETWIB_ERR_FUPTHREADATTRSETDETACHSTATE: *pc = "pthread_attr_setdetachstate"; break;
    case NETWIB_ERR_FUPTHREADCONDINIT: *pc = "pthread_cond_init"; break;
    case NETWIB_ERR_FUPTHREADCONDBROADCAST: *pc = "pthread_cond_broadcast"; break;
    case NETWIB_ERR_FUPTHREADCONDDESTROY: *pc = "pthread_cond_destroy"; break;
    case NETWIB_ERR_FUPTHREADCONDSIGNAL: *pc = "pthread_cond_signal"; break;
    case NETWIB_ERR_FUPTHREADCONDTIMEDWAIT: *pc = "pthread_cond_timedwait"; break;
    case NETWIB_ERR_FUPTHREADCONDWAIT: *pc = "pthread_cond_wait"; break;
    case NETWIB_ERR_FUPTHREADCREATE: *pc = "pthread_create"; break;
    case NETWIB_ERR_FUPTHREADEXIT: *pc = "pthread_exit"; break;
    case NETWIB_ERR_FUPTHREADGETEXPIRATIONNP: *pc = "pthread_get_expiration_np"; break;
    case NETWIB_ERR_FUPTHREADGETSPECIFIC: *pc = "pthread_getspecific"; break;
    case NETWIB_ERR_FUPTHREADJOIN: *pc = "pthread_join"; break;
    case NETWIB_ERR_FUPTHREADKEYCREATE: *pc = "pthread_key_create"; break;
    case NETWIB_ERR_FUPTHREADKEYDELETE: *pc = "pthread_key_delete"; break;
    case NETWIB_ERR_FUPTHREADMUTEXINIT: *pc = "pthread_mutex_init"; break;
    case NETWIB_ERR_FUPTHREADMUTEXDESTROY: *pc = "pthread_mutex_destroy"; break;
    case NETWIB_ERR_FUPTHREADMUTEXLOCK: *pc = "pthread_mutex_lock"; break;
    case NETWIB_ERR_FUPTHREADMUTEXTIMEDLOCK: *pc = "pthread_mutex_timedlock"; break;
    case NETWIB_ERR_FUPTHREADMUTEXTRYLOCK: *pc = "pthread_mutex_trylock"; break;
    case NETWIB_ERR_FUPTHREADMUTEXUNLOCK: *pc = "pthread_mutex_unlock"; break;
    case NETWIB_ERR_FUPTHREADRWLOCKINIT: *pc = "pthread_rwlock_init"; break;
    case NETWIB_ERR_FUPTHREADRWLOCKDESTROY: *pc = "pthread_rwlock_destroy"; break;
    case NETWIB_ERR_FUPTHREADRWLOCKRDLOCK: *pc = "pthread_rwlock_rdlock"; break;
    case NETWIB_ERR_FUPTHREADRWLOCKWRLOCK: *pc = "pthread_rwlock_wrlock"; break;
    case NETWIB_ERR_FUPTHREADRWLOCKTIMEDRDLOCK: *pc = "pthread_rwlock_timedrdlock"; break;
    case NETWIB_ERR_FUPTHREADRWLOCKTIMEDWRLOCK: *pc = "pthread_rwlock_timedwrlock"; break;
    case NETWIB_ERR_FUPTHREADRWLOCKTRYRDLOCK: *pc = "pthread_rwlock_tryrdlock"; break;
    case NETWIB_ERR_FUPTHREADRWLOCKTRYWRLOCK: *pc = "pthread_rwlock_trywrlock"; break;
    case NETWIB_ERR_FUPTHREADRWLOCKUNLOCK: *pc = "pthread_rwlock_unlock"; break;
    case NETWIB_ERR_FUPTHREADSETSPECIFIC: *pc = "pthread_setspecific"; break;
    case NETWIB_ERR_FUPUTMSG: *pc = "putmsg"; break;
    case NETWIB_ERR_FURAISE: *pc = "raise"; break;
    case NETWIB_ERR_FUREADFILE: *pc = "ReadFile"; break;
    case NETWIB_ERR_FUREADDIRR: *pc = "readdir_r"; break;
    case NETWIB_ERR_FUREADLINK: *pc = "readlink"; break;
    case NETWIB_ERR_FUREAD: *pc = "read"; break;
    case NETWIB_ERR_FUREALLOC: *pc = "realloc"; break;
    case NETWIB_ERR_FURECV: *pc = "recv"; break;
    case NETWIB_ERR_FURECVFROM: *pc = "recvfrom"; break;
    case NETWIB_ERR_FUREGCLOSEKEY: *pc = "RegCloseEx"; break;
    case NETWIB_ERR_FUREGCOMP: *pc = "regcomp"; break;
    case NETWIB_ERR_FUREGENUMKEY: *pc = "RegEnumKey"; break;
    case NETWIB_ERR_FUREGENUMVALUE: *pc = "RegEnumValue"; break;
    case NETWIB_ERR_FUREGOPENKEYEX: *pc = "RegOpenKeyEx"; break;
    case NETWIB_ERR_FUREGQUERYINFOKEY: *pc = "RegQueryInfoKey"; break;
    case NETWIB_ERR_FUREGQUERYVALUEEX: *pc = "RegQueryValueEx"; break;
    case NETWIB_ERR_FURELEASEMUTEX: *pc = "ReleaseMutex"; break;
    case NETWIB_ERR_FURENAME: *pc = "rename"; break;
    case NETWIB_ERR_FURESETEVENT: *pc = "ResetEvent"; break;
    case NETWIB_ERR_FURMDIR: *pc = "rmdir"; break;
    case NETWIB_ERR_FUSELECT: *pc = "select"; break;
    case NETWIB_ERR_FUSEND: *pc = "send"; break;
    case NETWIB_ERR_FUSENDTO: *pc = "sendto"; break;
    case NETWIB_ERR_FUSETCONSOLEMODE: *pc = "SetConsoleMode"; break;
    case NETWIB_ERR_FUSETEGID: *pc = "setguid"; break;
    case NETWIB_ERR_FUSETEUID: *pc = "seteuid"; break;
    case NETWIB_ERR_FUSETEVENT: *pc = "SetEvent"; break;
    case NETWIB_ERR_FUSETPRIORITY: *pc = "setpriority"; break;
    case NETWIB_ERR_FUSETSOCKOPT: *pc = "setsockopt"; break;
    case NETWIB_ERR_FUSETSTDHANDLE: *pc = "SetStdHandle"; break;
    case NETWIB_ERR_FUSETTHREADPRIORITY: *pc = "SetThreadPriority"; break;
    case NETWIB_ERR_FUSIGNAL: *pc = "signal"; break;
    case NETWIB_ERR_FUSNMPEXTENSIONINIT: *pc = "SnmpExtensionInit"; break;
    case NETWIB_ERR_FUSNMPEXTENSIONQUERY: *pc = "SnmpExtensionQuery"; break;
    case NETWIB_ERR_FUSOCKET: *pc = "socket"; break;
    case NETWIB_ERR_FUSTAT: *pc = "stat"; break;
    case NETWIB_ERR_FUSYMLINK: *pc = "symlink"; break;
    case NETWIB_ERR_FUSYSCTL: *pc = "sysctl"; break;
    case NETWIB_ERR_FUSYSTEM: *pc = "system"; break;
    case NETWIB_ERR_FUTCGETATTR: *pc = "tcgetattr"; break;
    case NETWIB_ERR_FUTCSETATTR: *pc = "tcsetattr"; break;
    case NETWIB_ERR_FUTERMINATEPROCESS: *pc = "TerminateProcess"; break;
    case NETWIB_ERR_FUTERMINATETHREAD: *pc = "TerminateThread"; break;
    case NETWIB_ERR_FUTLSALLOC: *pc = "PeekNamedPipe"; break;
    case NETWIB_ERR_FUTLSFREE: *pc = "TlsFree"; break;
    case NETWIB_ERR_FUTLSGETVALUE: *pc = "TlsGetValue"; break;
    case NETWIB_ERR_FUTLSSETVALUE: *pc = "TlsSetValue"; break;
    case NETWIB_ERR_FUUNLINK: *pc = "unlink"; break;
    case NETWIB_ERR_FUVIRTUALALLOC: *pc = "VirtualAlloc"; break;
    case NETWIB_ERR_FUVIRTUALFREE: *pc = "VirtualFree"; break;
    case NETWIB_ERR_FUVIRTUALPROTECT: *pc = "VirtualProtect"; break;
    case NETWIB_ERR_FUWAITFORSINGLEOBJECT: *pc = "WaitForSingleObject"; break;
    case NETWIB_ERR_FUWAITPID: *pc = "waitpid"; break;
    case NETWIB_ERR_FUWRITEFILE: *pc = "WriteFile"; break;
    case NETWIB_ERR_FUWRITE: *pc = "write"; break;
    case NETWIB_ERR_FUWSACLEANUP: *pc = "WSACleanup"; break;
    case NETWIB_ERR_FUWSASTARTUP: *pc = "WSAStartup"; break;
    case NETWIB_ERR_FUWSASTARTUPVER: *pc = "WSAStartup (bad Winsock version)"; break;
  }

  if (error > NETWIB_ERR_FU_BEGIN && error < NETWIB_ERR_FU_END) {
    *padderrorin = NETWIB_TRUE;
  } else {
    *padderrorin = NETWIB_FALSE;
  }

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
static netwib_err netwib_priv_err_append_herrno(netwib_uint32 varherrno,
                                                netwib_buf *pbuf)
{
  netwib_er(netwib_buf_append_fmt(pbuf, " hint: h_errno = %{uint32}",
                                  varherrno));

#if defined NETWIBDEF_SYSNAME_Unix
 #if NETWIBDEF_HAVEFUNC_HSTRERROR==1
  netwib_er(netwib_buf_append_text(" = ", pbuf));
  netwib_er(netwib_buf_append_text(hstrerror(varherrno), pbuf));
 #endif
#elif defined NETWIBDEF_SYSNAME_Windows
  /* there is no portable way to get Winsock error messages,
     so we do it here */
  {
    netwib_conststring pc;
    pc = NULL;
    switch (varherrno) {
      case WSAEINTR:
        pc = "Interrupted system call";
        break;
      case WSAEBADF:
        pc = "Bad file number";
        break;
      case WSAEACCES:
        pc = "Permission denied";
        break;
      case WSAEFAULT:
        pc = "Bad address";
        break;
      case WSAEINVAL:
        pc = "Invalid argument";
        break;
      case WSAEMFILE:
        pc = "Too many open sockets";
        break;
      case WSAEWOULDBLOCK:
        pc = "Operation would block";
        break;
      case WSAEINPROGRESS:
        pc = "Operation now in progress";
        break;
      case WSAEALREADY:
        pc = "Operation already in progress";
        break;
      case WSAENOTSOCK:
        pc = "Socket operation on non-socket";
        break;
      case WSAEDESTADDRREQ:
        pc = "Destination address required";
        break;
      case WSAEMSGSIZE:
        pc = "Message too long";
        break;
      case WSAEPROTOTYPE:
        pc = "Protocol wrong type for socket";
        break;
      case WSAENOPROTOOPT:
        pc = "Bad protocol option";
        break;
      case WSAEPROTONOSUPPORT:
        pc = "Protocol not supported";
        break;
      case WSAESOCKTNOSUPPORT:
        pc = "Socket type not supported";
        break;
      case WSAEOPNOTSUPP:
        pc = "Operation not supported on socket";
        break;
      case WSAEPFNOSUPPORT:
        pc = "Protocol family not supported";
        break;
      case WSAEAFNOSUPPORT:
        pc = "Address family not supported";
        break;
      case WSAEADDRINUSE:
        pc = "Address already in use";
        break;
      case WSAEADDRNOTAVAIL:
        pc = "Can't assign requested address";
        break;
      case WSAENETDOWN:
        pc = "Network is down";
        break;
      case WSAENETUNREACH:
        pc = "Network is unreachable";
        break;
      case WSAENETRESET:
        pc = "Net connection reset";
        break;
      case WSAECONNABORTED:
        pc = "Software caused connection abort";
        break;
      case WSAECONNRESET:
        pc = "Connection reset by peer";
        break;
      case WSAENOBUFS:
        pc = "No buffer space available";
        break;
      case WSAEISCONN:
        pc = "Socket is already connected";
        break;
      case WSAENOTCONN:
        pc = "Socket is not connected";
        break;
      case WSAESHUTDOWN:
        pc = "Can't send after socket shutdown";
        break;
      case WSAETOOMANYREFS:
        pc = "Too many references, can't splice";
        break;
      case WSAETIMEDOUT:
        pc = "Connection timed out";
        break;
      case WSAECONNREFUSED:
        pc = "Connection refused";
        break;
      case WSAELOOP:
        pc = "Too many levels of symbolic links";
        break;
      case WSAENAMETOOLONG:
        pc = "File name too long";
        break;
      case WSAEHOSTDOWN:
        pc = "Host is down";
        break;
      case WSAEHOSTUNREACH:
        pc = "No route to host";
        break;
      case WSAENOTEMPTY:
        pc = "Directory not empty";
        break;
      case WSAEPROCLIM:
        pc = "Too many processes";
        break;
      case WSAEUSERS:
        pc = "Too many users";
        break;
      case WSAEDQUOT:
        pc = "Disc quota exceeded";
        break;
      case WSAESTALE:
        pc = "Stale NFS file handle";
        break;
      case WSAEREMOTE:
        pc = "Too many levels of remote in path";
        break;
      case WSASYSNOTREADY:
        pc = "Network system is unavailable";
        break;
      case WSAVERNOTSUPPORTED:
        pc = "Winsock DLL version out of range";
        break;
      case WSANOTINITIALISED:
        pc = "WSAStartup not yet called";
        break;
      case WSAEDISCON:
        pc = "Graceful shutdown in progress";
        break;
      case WSAHOST_NOT_FOUND:
        pc = "Host not found";
        break;
      case WSATRY_AGAIN:
        pc = "Non-Authoritative Host not found";
        break;
      case WSANO_RECOVERY:
        pc = "Non-Recoverable errors: FORMERR, REFUSED, NOTIMP";
        break;
      case WSANO_DATA:
        pc = "No host data of that type was found";
        break;
    }
    if (pc != NULL) {
      netwib_er(netwib_buf_append_text(" = ", pbuf));
      netwib_er(netwib_buf_append_text(pc, pbuf));
    }
  }
#else
#error "Unknown value for NETWIBDEF_SYSNAME"
#endif

  netwib_er(netwib_buf_append_text("\n", pbuf));

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_err_syserrors(netwib_uint32 *perrno,
                                     netwib_uint32 *pherrno,
                                     netwib_uint32 *pgetlasterror)
{

  *perrno = errno;
#if NETWIBDEF_HAVEVAR_HERRNO==1
  *pherrno = h_errno;
#else
  *pherrno = 0;
#endif
#if defined NETWIBDEF_SYSNAME_Unix
  *pgetlasterror = 0;
#elif defined NETWIBDEF_SYSNAME_Windows
  *pgetlasterror = GetLastError();
  /* we do not use WSAGetLastError, because it is equivalent to h_errno */
#else
#error "Unknown value for NETWIBDEF_SYSNAME"
#endif

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
static netwib_err netwib_priv_err_append_err2(netwib_buf *pbuf)
{
  netwib_string pc;
  netwib_priv_glovars_t *ptr = &netwib_priv_glovars;

  netwib_er(netwib_buf_ref_string(&(ptr->errmsg), &pc));
  if (pc[0] != '\0') {
    netwib_er(netwib_buf_append_text(" hint: ", pbuf));
    netwib_er(netwib_buf_append_text(pc, pbuf));
    netwib_er(netwib_buf_append_text("\n", pbuf));
  }

  return(NETWIB_ERR_OK);
}
netwib_err netwib_priv_err_append_err(netwib_err error,
                                      netwib_uint32 varerrno,
                                      netwib_uint32 varherrno,
                                      netwib_uint32 vargetlasterror,
                                      netwib_err_encodetype encodetype,
                                      netwib_buf *pbuf)
{
  netwib_conststring perrstr;
  netwib_bool adderrorin, canuseglo;
  netwib_err ret;

  /* "Error xyz : " */
  if (encodetype == NETWIB_ERR_ENCODETYPE_NUMTEXT ||
      encodetype == NETWIB_ERR_ENCODETYPE_FULL) {
    netwib_er(netwib_buf_append_fmt(pbuf, "Error %{uint32} : ", error));
  }

  /* error string */
  netwib_er(netwib_priv_err_text(error, &perrstr, &adderrorin));
  if (perrstr == NULL) {
    netwib_er(netwib_buf_append_text("unknown error", pbuf));
  } else {
    if (adderrorin) {
      netwib_er(netwib_buf_append_text("error in ", pbuf));
    }
    netwib_er(netwib_buf_append_text(perrstr, pbuf));
    if (adderrorin) {
      netwib_er(netwib_buf_append_text("()", pbuf));
    }
  }

  if (encodetype != NETWIB_ERR_ENCODETYPE_FULL) {
    /* stop here */
    return(NETWIB_ERR_OK);
  }
  netwib_er(netwib_buf_append_text("\n", pbuf));

  /* errno */
  if (varerrno) {
    netwib_er(netwib_buf_append_fmt(pbuf, " hint: errno = %{uint32} = %s\n",
                                    varerrno, strerror(varerrno)));
  }

  /* GetLastError */
#if defined NETWIBDEF_SYSNAME_Unix
  vargetlasterror = 0;
#elif defined NETWIBDEF_SYSNAME_Windows
  if (vargetlasterror) {
    LPVOID lpMsgBuf;
    int bytenum;

    /* display error number */
    netwib_er(netwib_buf_append_fmt(pbuf, " hint: GetLastError = %{uint32}",
                                  vargetlasterror));
    /* try to get error string */
    bytenum = FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
                            FORMAT_MESSAGE_FROM_SYSTEM,
                            NULL, vargetlasterror,
                            MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                            (LPTSTR) & lpMsgBuf, 0, NULL);
    if (bytenum) {
      netwib_er(netwib_buf_append_text(" = ", pbuf));
      netwib_er(netwib_buf_append_text((char *)lpMsgBuf, pbuf));
      /* use LocalFree, because it's not allocated in netwib */
      LocalFree(lpMsgBuf);
    } else {
      netwib_er(netwib_buf_append_text("\n", pbuf));
    }
  }
#else
#error "Unknown value for NETWIBDEF_SYSNAME"
#endif

  /* h_errno */
  if (varherrno) {
    netwib_er(netwib_priv_err_append_herrno(varherrno, pbuf));
  }

  /* global error message */
  netwib_er(netwib_priv_glovars_canuse(&canuseglo));
  if (canuseglo) {
    netwib_er(netwib_priv_glovars_rdlock());
    ret = netwib_priv_err_append_err2(pbuf);
    netwib_er(netwib_priv_glovars_rdunlock());
    if (ret != NETWIB_ERR_OK) {
      return(ret);
    }
  }

  /* backtrace */
#if NETWIB_DEBUG_ERROR_BT==1
  if (error != NETWIB_ERR_OK) {
    netwib_string str;
    netwib_er(netwib_debug_backtrace_str(&str));
    netwib_er(netwib_buf_append_text(" backtrace: ", pbuf));
    netwib_er(netwib_buf_append_text(str, pbuf));
    netwib_er(netwib_buf_append_text("\n", pbuf));
    netwib_er(netwib_debug_str_close(&str));
  }
#endif

  /*get threadid */
#if NETWIB_DEBUG_ERROR_THREADID==1
  {
    netwib_uint32 threadid;

    netwib_er(netwib_debug_thread_id(&threadid));
    netwib_er(netwib_buf_append_fmt(pbuf, " thread: %{uint32}\n", threadid));
  }
#endif

  return(NETWIB_ERR_OK);
}
