/* This file is part of Netwib.
*/

/*-------------------------------------------------------------*/
netwib_err netwib_priv_rand_seed(netwib_uint32 *pseed)
{

#if defined NETWIBDEF_SYSNAME_Unix
  {
    int fd, reti;
    char arr[4];
    netwib_err ret;

    /* try with "/dev/random" */
    fd = open("/dev/random", O_RDONLY);
    if (fd != -1) {
      ret = netwib_priv_fd_block_set(fd, NETWIB_FALSE);
      if (ret == NETWIB_ERR_OK) {
        reti = read(fd, arr, 4);
        if (reti == 4) {
          if (pseed != NULL) {
            *pseed = netwib_c2_uint32_4(arr[0], arr[1], arr[2], arr[3]);
          }
          close(fd);
          errno = 0;
          return(NETWIB_ERR_OK);
        }
      }
      close(fd);
    }

    /* try with "/dev/urandom" */
    fd = open("/dev/urandom", O_RDONLY);
    if (fd != -1) {
      ret = netwib_priv_fd_block_set(fd, NETWIB_FALSE);
      if (ret == NETWIB_ERR_OK) {
        reti = read(fd, arr, 4);
        if (reti == 4) {
          if (pseed != NULL) {
            *pseed = netwib_c2_uint32_4(arr[0], arr[1], arr[2], arr[3]);
          }
          close(fd);
          errno = 0;
          return(NETWIB_ERR_OK);
        }
      }
      close(fd);
    }

    /* use time and pid */
    if (pseed != NULL) {
      *pseed = time(0) ^ ((getpid() << 16) | getppid());
    }
  }
#elif defined NETWIBDEF_SYSNAME_Windows
  {
    struct _timeb tstruct;

    /* use localtime */
    _ftime(&tstruct);
    if (pseed != NULL) {
      *pseed = tstruct.time ^ tstruct.millitm;
    }
  }
#else
#error "Unknown value for NETWIBDEF_SYSNAME"
#endif

  errno = 0;
  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_uint32_init_rand(netwib_uint32 *prand)
{
  netwib_uint32 therand;
  netwib_priv_glovars_t *ptr = &netwib_priv_glovars;
  netwib_int32 fortest;

  /*
     Random generation using linear congruential generators
     T+1 = ( aT ) % m           (may overflow)
     = ( a * (T%q)) - rT/q  (doesn't overflow)
     with :
     a = 16807
     m = 2147483647
     q = (m / a) = 127773
     r = (m % a) = 2836
   */
  netwib_er(netwib_priv_glovars_wrlock());
  fortest = 16807 * (ptr->rand_seed % 127773)
            - 2836 * (ptr->rand_seed / 127773);
  therand = fortest;
  if (fortest <= 0) {
    therand += 2147483647;
  }
  ptr->rand_seed = therand;
  netwib_er(netwib_priv_glovars_wrunlock());

  if (prand != NULL) {
    *prand = therand;
  }

  return(NETWIB_ERR_OK);
}
