/*
                                  NETWOX
                             Network toolbox
                Copyright(c) 1999-2004 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Netwox.

  Netwox is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  version 2 as published by the Free Software Foundation.

  Netwox is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

/*-------------------------------------------------------------*/
#include "../netwox.h"

/*-------------------------------------------------------------*/
netwib_err netwox_nntp_write_modereader(netwib_io *pio)
{
  netwib_buf buf;
  netwib_err ret;

  netwib_er(netwib_buf_init_ext_text("MODE READER\r\n", &buf));
  ret = netwib_io_write(pio, &buf);
  return(ret);
}

/*-------------------------------------------------------------*/
netwib_err netwox_nntp_write_group(netwib_io *pio,
                                   netwib_constbuf *pnewsgroup)
{
  netwib_byte array[512];
  netwib_buf buf;
  netwib_err ret;

  netwib_er(netwib_buf_init_ext_arrayempty(array, sizeof(array), &buf));
  buf.flags |= NETWIB_BUF_FLAGS_CANALLOC;

  netwib_er(netwib_buf_append_text("GROUP ", &buf));
  netwib_er(netwib_buf_append_buf(pnewsgroup, &buf));
  netwib_er(netwib_buf_append_text("\r\n", &buf));

  ret = netwib_io_write(pio, &buf);

  netwib_er(netwib_buf_close(&buf));

  return(ret);
}

/*-------------------------------------------------------------*/
netwib_err netwox_nntp_write_article(netwib_io *pio,
                                     netwib_uint32 artnum)
{
  netwib_byte array[512];
  netwib_buf buf;
  netwib_err ret;

  netwib_er(netwib_buf_init_ext_arrayempty(array, sizeof(array), &buf));
  buf.flags |= NETWIB_BUF_FLAGS_CANALLOC;

  netwib_er(netwib_buf_append_fmt(&buf, "ARTICLE %{uint32}\r\n", artnum));

  ret = netwib_io_write(pio, &buf);

  netwib_er(netwib_buf_close(&buf));

  return(ret);
}

/*-------------------------------------------------------------*/
netwib_err netwox_nntp_write_list(netwib_io *pio)
{
  netwib_buf buf;
  netwib_err ret;

  netwib_er(netwib_buf_init_ext_text("LIST\r\n", &buf));
  ret = netwib_io_write(pio, &buf);
  return(ret);
}

/*-------------------------------------------------------------*/
netwib_err netwox_nntp_write_quit(netwib_io *pio)
{
  netwib_buf buf;
  netwib_err ret;

  netwib_er(netwib_buf_init_ext_text("QUIT\r\n", &buf));
  ret = netwib_io_write(pio, &buf);
  return(ret);
}

/*-------------------------------------------------------------*/
netwib_err netwox_nntp_write_post_begin(netwib_io *pio)
{
  netwib_buf buf;

  netwib_er(netwib_buf_init_ext_text("POST\r\n", &buf));
  netwib_er(netwib_io_write(pio, &buf));

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwox_nntp_post(netwib_bufpool *pbufpool,
                            netwib_io *pio,
                            netwib_uint32 maxwaitms,
                            netwib_constbuf *pnewsgroups,
                            netwib_constbuf *pdata)
{
  netwib_buf *perrmsg;
  netwib_uint32 errnum;
  netwib_err ret=NETWIB_ERR_OK;

  netwib_er(netwib_bufpool_buf_init(pbufpool, &perrmsg));

  netwib_eg(netwox_txtproto_read_reply(pio, maxwaitms, &errnum, perrmsg));
  netwib_eg(netwox_txtproto_expect1(errnum, perrmsg,
                                    NETWOX_NNTP_REPLYCODE_SVCREADY));

  netwib_eg(netwox_nntp_write_modereader(pio));
  netwib__buf_reinit(perrmsg);
  netwib_eg(netwox_txtproto_read_reply(pio, maxwaitms, &errnum,perrmsg));
  netwib_eg(netwox_txtproto_expect1(errnum, perrmsg,
                                    NETWOX_NNTP_REPLYCODE_SVCREADY));

  netwib_eg(netwox_nntp_write_group(pio, pnewsgroups));
  netwib__buf_reinit(perrmsg);
  netwib_eg(netwox_txtproto_read_reply(pio, maxwaitms, &errnum, perrmsg));
  netwib_eg(netwox_txtproto_expect1(errnum, perrmsg,
                                    NETWOX_NNTP_REPLYCODE_GROUPSELECTED));

  netwib_eg(netwox_nntp_write_post_begin(pio));
  netwib__buf_reinit(perrmsg);
  netwib_eg(netwox_txtproto_read_reply(pio, maxwaitms, &errnum, perrmsg));
  netwib_eg(netwox_txtproto_expect1(errnum, perrmsg,
                                    NETWOX_NNTP_REPLYCODE_SENDDATA));

  netwib_eg(netwox_nntp_write_post_content(pio, pdata));
  netwib_eg(netwox_nntp_write_post_end(pio));
  netwib__buf_reinit(perrmsg);
  netwib_eg(netwox_txtproto_read_reply(pio, maxwaitms, &errnum, perrmsg));
  netwib_eg(netwox_txtproto_expect1(errnum, perrmsg,
                                    NETWOX_NNTP_REPLYCODE_POSTOK));

  netwib_eg(netwox_nntp_write_quit(pio));
  netwib__buf_reinit(perrmsg);
  netwib_eg(netwox_txtproto_read_reply(pio, maxwaitms, &errnum, perrmsg));
  netwib_eg(netwox_txtproto_expect1(errnum, perrmsg,
                                    NETWOX_NNTP_REPLYCODE_SVCCLOSING));

 netwib_gotolabel:
  netwib_er(netwib_bufpool_buf_close(pbufpool, &perrmsg));
  return(ret);
}

/*-------------------------------------------------------------*/
netwib_err netwox_nntp_download(netwib_bufpool *pbufpool,
                                netwib_io *pio,
                                netwib_uint32 maxwaitms,
                                netwib_uint32 artnum,
                                netwib_constbuf *pfilename)
{
  netwib_buf *perrmsg, *pbuf;
  netwib_bool event;
  netwib_data data;
  netwib_time t;
  netwib_uint32 errnum, datasize;
  netwib_io *pfileio;
  netwib_err ret=NETWIB_ERR_OK;

  netwib_er(netwox_nntp_write_article(pio, artnum));
  netwib_er(netwib_bufpool_buf_init(pbufpool, &perrmsg));
  netwib_er(netwox_txtproto_read_reply(pio, maxwaitms, &errnum, perrmsg));
  switch(errnum) {
  case NETWOX_NNTP_REPLYCODE_ARTICLEHEADBODY :
    ret = NETWIB_ERR_OK;
    break;
  case NETWOX_NNTP_REPLYCODE_NOARTICLENUMBER :
  case NETWOX_NNTP_REPLYCODE_NOARTICLEFOUND :
    ret = NETWIB_ERR_OKNOTFOUND;
    break;
  default:
    ret = netwox_txtproto_expect1(errnum, perrmsg,
                                  NETWOX_NNTP_REPLYCODE_ARTICLEHEADBODY);
    break;
  }
  netwib_er(netwib_bufpool_buf_close(pbufpool, &perrmsg));
  if (ret != NETWIB_ERR_OK) {
    return(ret);
  }

  netwib_er(netwib_bufpool_buf_init(pbufpool, &pbuf));
  netwib_er(netwib_io_init_file_write(pfilename, &pfileio));
  while(NETWIB_TRUE) {
    /* read */
    netwib__buf_reinit(pbuf);
    netwib_er(netwib_time_init_now(&t));
    netwib_er(netwib_time_plus_msec(&t, maxwaitms));
    netwib_er(netwib_io_wait_read(pio, &t, &event));
    if (!event) {
      break;
    }
    ret = netwib_io_read(pio, pbuf);
    if (ret == NETWIB_ERR_OKDATANOTAVAIL) {
      continue;
    } else if (ret != NETWIB_ERR_OK) {
      break;
    }
    /* check dots */
    datasize = netwib__buf_ref_data_size(pbuf);
    if (datasize > 0) {
      data = netwib__buf_ref_data_ptr(pbuf);
      if (data[0] == '.') {
        if (datasize == 1) {
          /* a dot alone indicates end */
          break;
        } else {
          /* a dot in the message is duplicated */
          pbuf->beginoffset++;
        }
      }
    }
    /* append to file */
    netwib_er(netwib_buf_append_text("\r\n", pbuf));
    netwib_er(netwib_io_write(pfileio, pbuf));
  }
  netwib_er(netwib_io_close(&pfileio));
  netwib_er(netwib_bufpool_buf_close(pbufpool, &pbuf));

  return(ret);
}

