/*
                                  NETWOX
                             Network toolbox
                Copyright(c) 1999-2004 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Netwox.

  Netwox is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  version 2 as published by the Free Software Foundation.

  Netwox is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

/*-------------------------------------------------------------*/
#include "../netwox.h"

/*-------------------------------------------------------------*/
typedef struct {
  netwib_buf bufrd;
  netwib_buf bufwr;
} netwox_telnet;

/*-------------------------------------------------------------*/
typedef enum {
  NETWOX_TELNET_OPT_ECHO =   1,
  NETWOX_TELNET_OPT_SGA =    3, /* suppress go ahead */
  NETWOX_TELNET_OPT_SE =   240, /* end of subnegotiation */
  NETWOX_TELNET_OPT_NOP =  241, /* no operation */
  NETWOX_TELNET_OPT_DM =   242, /* data mark */
  NETWOX_TELNET_OPT_BRK =  243, /* break */
  NETWOX_TELNET_OPT_IP =   244, /* interrupt process */
  NETWOX_TELNET_OPT_AO =   245, /* abort output */
  NETWOX_TELNET_OPT_AYT =  246, /* are you there */
  NETWOX_TELNET_OPT_EC =   247, /* erase character */
  NETWOX_TELNET_OPT_EL =   248, /* erase line */
  NETWOX_TELNET_OPT_GA =   249, /* go ahead */
  NETWOX_TELNET_OPT_SB =   250, /* sub negotiation */
  NETWOX_TELNET_OPT_WILL = 251,
  NETWOX_TELNET_OPT_WONT = 252,
  NETWOX_TELNET_OPT_DO =   253,
  NETWOX_TELNET_OPT_DONT = 254,
  NETWOX_TELNET_OPT_IAC =  255  /* interpret as command */
} netwox_telnet_opt;

/*-------------------------------------------------------------*/
/* Note : as we reject every option, subnegotiation is not used (rfc855).
   So, we only have fixed bytes commands "IAC option", and not variable
   length commands "IAC SB option parameters IAC SE". */

/*-------------------------------------------------------------*/
static netwib_err netwox_telnet_init2(netwib_bool *preadinitialized,
                                      netwib_bool *pwriteinitialized,
                                      netwox_telnet *ptr)
{

  /* initialize structures */
  netwib_er(netwib_buf_init_mallocdefault(&ptr->bufrd));
  ptr->bufrd.flags |= NETWIB_BUF_FLAGS_CANSLIDE;
  netwib_er(netwib_buf_init_mallocdefault(&ptr->bufwr));

  /* set io values */
  *preadinitialized = NETWIB_TRUE;
  *pwriteinitialized = NETWIB_TRUE;

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
static netwib_err netwox_telnet_close(netwox_telnet *ptr)
{

  /* close structures */
  netwib_er(netwib_buf_close(&ptr->bufwr));
  netwib_er(netwib_buf_close(&ptr->bufrd));

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
static netwib_err netwox_telnet_decode_opt(netwox_telnet_opt opt,
                                           netwib_constdata data,
                                           netwib_uint32 datasize,
                                           netwib_io *pnextwrio,
                                           netwib_uint32 *pskipsize)
{
  netwib_byte array[3];
  netwib_buf buf;
  netwox_telnet_opt serveropt;

#if 0
  if (datasize < 3) {
    netwib_er(netwib_fmt_display("Too short\n"));
  } else {
    serveropt = data[2];
    switch(opt) {
    case NETWOX_TELNET_OPT_DO :
      netwib_er(netwib_fmt_display("TEL DO %{uint32:02X}\n", serveropt));
      break;
    case NETWOX_TELNET_OPT_DONT :
      netwib_er(netwib_fmt_display("TEL DONT %{uint32:02X}\n", serveropt));
      break;
    case NETWOX_TELNET_OPT_WILL :
      netwib_er(netwib_fmt_display("TEL WILL %{uint32:02X}\n", serveropt));
      break;
    case NETWOX_TELNET_OPT_WONT :
      netwib_er(netwib_fmt_display("TEL WONT %{uint32:02X}\n", serveropt));
      break;
    default :
      netwib_er(netwib_fmt_display("TEL UNKNOWN %{uint32:02X}\n", opt));
      break;
    }
  }
#endif

  switch(opt) {
    case NETWOX_TELNET_OPT_NOP :
      *pskipsize = 2;
      break;
    case NETWOX_TELNET_OPT_DO :
    case NETWOX_TELNET_OPT_DONT :
      if (datasize < 3) {
        *pskipsize = 0;
      }
      serveropt = data[2];
      /* answer WON'T */
      array[0] = (netwib_byte)NETWOX_TELNET_OPT_IAC;
      array[1] = (netwib_byte)NETWOX_TELNET_OPT_WONT;
      array[2] = (netwib_byte)serveropt;
      netwib_er(netwib_buf_init_ext_arrayfilled(array, 3, &buf));
      netwib_er(netwib_io_write(pnextwrio, &buf));
      *pskipsize = 3;
      break;
    case NETWOX_TELNET_OPT_WILL :
    case NETWOX_TELNET_OPT_WONT :
      if (datasize < 3) {
        *pskipsize = 0;
      }
      serveropt = data[2];
      /* answer DON'T most of the time */
      array[0] = (netwib_byte)NETWOX_TELNET_OPT_IAC;
      if (opt == NETWOX_TELNET_OPT_WILL &&
          (serveropt == NETWOX_TELNET_OPT_ECHO ||
           serveropt == NETWOX_TELNET_OPT_SGA)) {
        array[1] = (netwib_byte)NETWOX_TELNET_OPT_DO;
      } else {
        array[1] = (netwib_byte)NETWOX_TELNET_OPT_DONT;
      }
      array[2] = (netwib_byte)serveropt;
      netwib_er(netwib_buf_init_ext_arrayfilled(array, 3, &buf));
      netwib_er(netwib_io_write(pnextwrio, &buf));
      *pskipsize = 3;
      break;
    default :
      netwib_er(netwib_fmt_display("Warning : don't know how to decode option %{uint32:02X}\n", opt));
      *pskipsize = 2;
      break;
  }

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
static netwib_err netwox_telnet_read(netwib_io *pio,
                                     netwib_buf *pbuf)
{
  netwox_telnet *ptr = (netwox_telnet *)pio->pcommon;
  netwib_data data, pc;
  netwib_uint32 datasize, initialdatasize, chunksize, oldendoffset, skipsize=0;
  netwox_telnet_opt opt;
  netwib_byte iacarray[2];

  iacarray[0] = (netwib_byte)NETWOX_TELNET_OPT_IAC;

  /* read */
  netwib_er(netwib_io_read(pio->rd.pnext, &ptr->bufrd));

  /* check for options (answer to them) and data */
  data = netwib__buf_ref_data_ptr(&ptr->bufrd);
  datasize = netwib__buf_ref_data_size(&ptr->bufrd);
  initialdatasize = datasize;
  oldendoffset = pbuf->endoffset;
  while (datasize) {
    pc = netwib_c_memchr(data, NETWOX_TELNET_OPT_IAC, datasize);
    if (pc == NULL) {
      /* give last part to user */
      netwib_er(netwib_buf_append_data(data, datasize, pbuf));
      datasize = 0;
      netwib__buf_reinit(&ptr->bufrd);
    } else {
      /* give chunk to user */
      chunksize = pc - data;
      netwib_er(netwib_buf_append_data(data, chunksize, pbuf));
      /* skip chunk */
      data += chunksize;
      datasize -= chunksize;
      if (datasize == 1) {
        /* no sufficient data */
        break;
      }
      /* obtain option */
      opt = data[1];
      if (opt == NETWOX_TELNET_OPT_IAC) {
        /* convert it to 0xFF char (was duplicated) */
                netwib_er(netwib_buf_append_data(iacarray, 1, pbuf));
        skipsize = 2;
      } else {
        netwib_er(netwox_telnet_decode_opt(opt, data, datasize, pio->wr.pnext,
                                           &skipsize));
        if (skipsize == 0) {
          /* no sufficient data */
          break;
        }
      }
      /* skip option */
      data += skipsize;
      datasize -= skipsize;
    }
  }

  if (datasize) {
    /* we will need to read more. For the moment return */
    ptr->bufrd.beginoffset += initialdatasize - datasize;
    if (pbuf->endoffset == oldendoffset) {
      return(NETWIB_ERR_OKDATANOTAVAIL);
    }
  }

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
static netwib_err netwox_telnet_write(netwib_io *pio,
                                      netwib_constbuf *pbuf)
{
  netwox_telnet *ptr = (netwox_telnet *)pio->pcommon;
  netwib_data data, pc;
  netwib_uint32 datasize, chunksize;
  netwib_byte iacarray[2];

  iacarray[0] = (netwib_byte)NETWOX_TELNET_OPT_IAC;

  /* only duplicate 0xFF/NETWOX_TELNET_OPT_IAC in user data */
  netwib__buf_reinit(&ptr->bufwr);
  data = netwib__buf_ref_data_ptr(pbuf);
  datasize = netwib__buf_ref_data_size(pbuf);
  while (datasize) {
    pc = netwib_c_memchr(data, NETWOX_TELNET_OPT_IAC, datasize);
    if (pc == NULL) {
      netwib_er(netwib_buf_append_data(data, datasize, &ptr->bufwr));
      datasize = 0;
    } else {
      chunksize = pc - data + 1;
      netwib_er(netwib_buf_append_data(data, chunksize, &ptr->bufwr));
      netwib_er(netwib_buf_append_data(iacarray, 1, &ptr->bufwr));
      data += chunksize;
      datasize -= chunksize;
    }
  }

  /* write */
  netwib_er(netwib_io_write(pio->wr.pnext, &ptr->bufwr));

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
static netwib_err netwox_telnet_fclose(netwib_io *pio)
{
  netwox_telnet *ptr = (netwox_telnet *)pio->pcommon;
  netwib_er(netwox_telnet_close(ptr));
  netwib_er(netwib_ptr_free(&pio->pcommon));
  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwox_telnet_init(netwib_io **ppio)
{
  netwib_bool rdinit, wrinit;
  netwib_ptr pcommon;
  netwib_err ret;

  netwib_er(netwib_ptr_malloc(sizeof(netwox_telnet), &pcommon));
  ret = netwox_telnet_init2(&rdinit, &wrinit, (netwox_telnet *)pcommon);
  if (ret != NETWIB_ERR_OK) {
    netwib_er(netwib_ptr_free(&pcommon));
    return(ret);
  }

  netwib_er(netwib_io_init(rdinit, wrinit,
                           pcommon,
                           &netwox_telnet_read,
                           &netwox_telnet_write,
                           NULL, /* wait */
                           NULL, /* unread */
                           NULL, /* set */
                           NULL, /* get */
                           &netwox_telnet_fclose,
                           ppio));

  return(NETWIB_ERR_OK);
}
