/*
                                  NETWOX
                             Network toolbox
                Copyright(c) 1999-2004 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Netwox.

  Netwox is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  version 2 as published by the Free Software Foundation.

  Netwox is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

/*-------------------------------------------------------------*/
#include "../netwox.h"

/*-------------------------------------------------------------*/
netwox_toolarg t000027_args[] = {
  NETWOX_TOOLARG_REQ_BUF_FILE_RD('f', NULL, NULL, NULL),
  NETWOX_TOOLARG_END
};
netwox_tooltreenodetype t000027_nodes[] = {
  NETWOX_TOOLTREENODETYPE_NOTNET_FILE,
  NETWOX_TOOLTREENODETYPE_END
};
netwox_tool_info t000027_info = {
  "Compute MD5 of a file",
  "",
  t000027_args,
  t000027_nodes,
};

/*-------------------------------------------------------------*/
static netwib_err t000027_loop(netwib_constbuf *pfilename)
{
  netwib_buf buf;
  netwib_io *pio;
  netwox_md5_context ctx;
  netwib_err ret=NETWIB_ERR_OK;

  /* initialize */
  netwib_er(netwib_io_init_file_read(pfilename, &pio));
  netwib_er(netwox_md5_init(&ctx));

  /* main loop */
  netwib_er(netwib_buf_init_mallocdefault(&buf));
  while (NETWIB_TRUE) {
    /* read */
    netwib__buf_reinit(&buf);
    ret = netwib_io_read(pio, &buf);
    if (ret == NETWIB_ERR_OKDATANOTAVAIL) {
      continue;
    } else if (ret == NETWIB_ERR_OKDATAEND) {
      ret = NETWIB_ERR_OK;
      break;
    } else if (ret != NETWIB_ERR_OK) {
      break;
    }
    /* update */
    netwib_er(netwox_md5_update(&ctx, &buf));
  }


  if (ret == NETWIB_ERR_OK) {
    netwib__buf_reinit(&buf);
    netwib_er(netwox_md5_final_hexa(&ctx, &buf));
    netwib_er(netwib_fmt_display("%{buf}\n", &buf));
  }

  /* close */
  netwib_er(netwib_buf_close(&buf));
  netwib_er(netwib_io_close(&pio));

  return(ret);
}

/*-------------------------------------------------------------*/
netwib_err t000027_core(int argc, char *argv[])
{
  netwox_arg *parg;
  netwib_buf filename;

  /* obtain parameters */
  netwib_er(netwox_arg_init(argc, argv, &t000027_info, &parg));
  netwib_er(netwox_arg_buf(parg, 'f', &filename));

  netwib_er(t000027_loop(&filename));

  netwib_er(netwox_arg_close(&parg));

  return(NETWIB_ERR_OK);
}
