/*
                                  NETWOX
                             Network toolbox
                Copyright(c) 1999-2004 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Netwox.

  Netwox is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  version 2 as published by the Free Software Foundation.

  Netwox is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

/*-------------------------------------------------------------*/
#include "../netwox.h"

/*-------------------------------------------------------------*/
netwox_toolarg t000108_args[] = {
  NETWOX_SOCK_ARG_TCP_CLIPORT("119"),
  NETWOX_TOOLARG_END
};
netwox_tooltreenodetype t000108_nodes[] = {
  NETWOX_TOOLTREENODETYPE_CLIENT_TCP_NNTP,
  NETWOX_TOOLTREENODETYPE_END
};
netwox_tool_info t000108_info = {
  "List newsgroups available on a server",
  "",
  t000108_args,
  t000108_nodes,
};

/*-------------------------------------------------------------*/
static netwib_err t000108_list2(netwib_io *pio,
                                netwib_uint32 maxwaitms)
{
  netwib_buf buf;
  netwib_bool event;
  netwib_data data;
  netwib_time t;
  netwib_err ret=NETWIB_ERR_OK;

  netwib_er(netwib_buf_init_mallocdefault(&buf));
  while(NETWIB_TRUE) {
    /* read */
    netwib__buf_reinit(&buf);
    netwib_er(netwib_time_init_now(&t));
    netwib_er(netwib_time_plus_msec(&t, maxwaitms));
    netwib_er(netwib_io_wait_read(pio, &t, &event));
    if (!event) {
      break;
    }
    ret = netwib_io_read(pio, &buf);
    if (ret == NETWIB_ERR_OKDATANOTAVAIL) {
      continue;
    } else if (ret != NETWIB_ERR_OK) {
      break;
    }
    if (netwib__buf_ref_data_size(&buf) == 1) {
      data = netwib__buf_ref_data_ptr(&buf);
      if (data[0] == '.') {
        break;
      }
    }
    /* display line */
    netwib_er(netwib_fmt_display("%{buf}\n", &buf));
  }
  netwib_er(netwib_buf_close(&buf));

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
static netwib_err t000108_list(netwib_io *pio,
                               netwib_uint32 maxwaitms)
{
  netwib_bufpool *pbufpool;
  netwib_buf *perrmsg;
  netwib_uint32 errnum;

  netwib_er(netwib_bufpool_initdefault(&pbufpool));
  netwib_er(netwib_bufpool_buf_init(pbufpool, &perrmsg));

  /* prepare */
  netwib_er(netwox_txtproto_read_reply(pio, maxwaitms, &errnum, perrmsg));
  netwib_er(netwox_txtproto_expect2(errnum, perrmsg,
                                    NETWOX_NNTP_REPLYCODE_SVCREADY,
                                    NETWOX_NNTP_REPLYCODE_SVCREADYNOPOST));

  netwib_er(netwox_nntp_write_modereader(pio));
  netwib__buf_reinit(perrmsg);
  netwib_er(netwox_txtproto_read_reply(pio, maxwaitms, &errnum, perrmsg));
  netwib_er(netwox_txtproto_expect1(errnum, perrmsg,
                                    NETWOX_NNTP_REPLYCODE_SVCREADY));

  netwib_er(netwox_nntp_write_list(pio));
  netwib__buf_reinit(perrmsg);
  netwib_er(netwox_txtproto_read_reply(pio, maxwaitms, &errnum, perrmsg));
  netwib_er(netwox_txtproto_expect1(errnum, perrmsg,
                                    NETWOX_NNTP_REPLYCODE_LISTFOLLOWS));

  /* read list */
  netwib_er(t000108_list2(pio, maxwaitms));

  /* leave */
  netwib_er(netwox_nntp_write_quit(pio));
  netwib__buf_reinit(perrmsg);
  netwib_er(netwox_txtproto_read_reply(pio, maxwaitms, &errnum, perrmsg));
  netwib_er(netwox_txtproto_expect1(errnum, perrmsg,
                                    NETWOX_NNTP_REPLYCODE_SVCCLOSING));

  netwib_er(netwib_bufpool_buf_close(pbufpool, &perrmsg));
  netwib_er(netwib_bufpool_close(&pbufpool));
  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err t000108_core(int argc, char *argv[])
{
  netwox_arg *parg;
  netwox_sockinfo sockinfo;
  netwib_io *pioserver, *pioline;

  /* obtain parameters */
  netwib_er(netwox_arg_init(argc, argv, &t000108_info, &parg));

  /* connect to server */
  netwib_er(netwox_sockinfo_init_arg_tcp_cli(parg, &sockinfo));
  netwib_er(netwox_sock_init(&sockinfo, &pioserver));

  /* list groups */
  netwib_er(netwox_txtproto_io_rd(pioserver, &pioline));
  netwib_er(t000108_list(pioline, 10000));

  /* close */
  netwib_er(netwib_io_close(&pioline));
  netwib_er(netwox_sockinfo_close(&sockinfo));
  netwib_er(netwox_arg_close(&parg));

  return(NETWIB_ERR_OK);
}
