/*
                                  NETWOX
                             Network toolbox
                Copyright(c) 1999-2004 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Netwox.

  Netwox is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  version 2 as published by the Free Software Foundation.

  Netwox is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

/*-------------------------------------------------------------*/
#include "../netwox.h"

/*-------------------------------------------------------------*/
netwox_toolarg t000109_args[] = {
  NETWOX_SOCK_ARG_TCP_CLIPORT("119"),
  NETWOX_TOOLARG_REQ_BUF('g', "newsgroup", "newsgroup", "comp.abc.def"),
  NETWOX_TOOLARG_REQ_UINT32('n', "artnum", "starting article number", "123"),
  NETWOX_TOOLARG_END
};
netwox_tooltreenodetype t000109_nodes[] = {
  NETWOX_TOOLTREENODETYPE_CLIENT_TCP_NNTP,
  NETWOX_TOOLTREENODETYPE_END
};
netwox_tool_info t000109_info = {
  "Download one, or more, newsgroup messages",
  "",
  t000109_args,
  t000109_nodes,
};

/*-------------------------------------------------------------*/
static netwib_err t000109_down(netwib_io *pio,
                               netwib_uint32 maxwaitms,
                               netwib_constbuf *pnewsgroup,
                               netwib_uint32 startartnum)
{
  netwib_bufpool *pbufpool;
  netwib_buf *perrmsg, filename;
  netwib_uint32 errnum, artnum;
  netwib_err ret;

  netwib_er(netwib_bufpool_initdefault(&pbufpool));
  netwib_er(netwib_bufpool_buf_init(pbufpool, &perrmsg));

  /* prepare */
  netwib_er(netwox_txtproto_read_reply(pio, maxwaitms, &errnum, perrmsg));
  netwib_er(netwox_txtproto_expect2(errnum, perrmsg,
                                    NETWOX_NNTP_REPLYCODE_SVCREADY,
                                    NETWOX_NNTP_REPLYCODE_SVCREADYNOPOST));

  netwib_er(netwox_nntp_write_modereader(pio));
  netwib__buf_reinit(perrmsg);
  netwib_er(netwox_txtproto_read_reply(pio, maxwaitms, &errnum, perrmsg));
  netwib_er(netwox_txtproto_expect1(errnum, perrmsg,
                                    NETWOX_NNTP_REPLYCODE_SVCREADY));

  netwib_er(netwox_nntp_write_group(pio, pnewsgroup));
  netwib__buf_reinit(perrmsg);
  netwib_er(netwox_txtproto_read_reply(pio, maxwaitms, &errnum, perrmsg));
  netwib_er(netwox_txtproto_expect1(errnum, perrmsg,
                                    NETWOX_NNTP_REPLYCODE_GROUPSELECTED));

  /* download messages */
  netwib_er(netwib_buf_init_mallocdefault(&filename));
  ret = NETWIB_ERR_OK;
  for (artnum = startartnum; ; artnum++) {
    netwib__buf_reinit(&filename);
    netwib_er(netwib_buf_append_fmt(&filename, "%{buf}_%{uint32}.txt",
                                    pnewsgroup, artnum));
    ret = netwox_nntp_download(pbufpool, pio, maxwaitms, artnum, &filename);
    if (ret != NETWIB_ERR_OK) {
      if (ret == NETWIB_ERR_OKNOTFOUND) ret = NETWIB_ERR_OK;
      break;
    }
  }
  netwib_er(netwib_buf_close(&filename));

  /* leave */
  netwib_er(netwox_nntp_write_quit(pio));
  netwib__buf_reinit(perrmsg);
  netwib_er(netwox_txtproto_read_reply(pio, maxwaitms, &errnum, perrmsg));
  netwib_er(netwox_txtproto_expect1(errnum, perrmsg,
                                    NETWOX_NNTP_REPLYCODE_SVCCLOSING));

  netwib_er(netwib_bufpool_buf_close(pbufpool, &perrmsg));
  netwib_er(netwib_bufpool_close(&pbufpool));
  return(ret);
}

/*-------------------------------------------------------------*/
netwib_err t000109_core(int argc, char *argv[])
{
  netwox_arg *parg;
  netwox_sockinfo sockinfo;
  netwib_buf newsgroup;
  netwib_uint32 startartnum;
  netwib_io *pioserver, *pioline;

  /* obtain parameters */
  netwib_er(netwox_arg_init(argc, argv, &t000109_info, &parg));
  netwib_er(netwox_arg_buf(parg, 'g', &newsgroup));
  netwib_er(netwox_arg_uint32(parg, 'n', &startartnum));

  /* connect to server */
  netwib_er(netwox_sockinfo_init_arg_tcp_cli(parg, &sockinfo));
  netwib_er(netwox_sock_init(&sockinfo, &pioserver));

  /* download messages */
  netwib_er(netwox_txtproto_io_rd(pioserver, &pioline));
  netwib_er(t000109_down(pioline, 10000, &newsgroup, startartnum));

  /* close */
  netwib_er(netwib_io_close(&pioline));
  netwib_er(netwox_sockinfo_close(&sockinfo));
  netwib_er(netwox_arg_close(&parg));

  return(NETWIB_ERR_OK);
}
